@@ -334,7 +334,7 @@ gboolean change_dir(gchar * filename)
 	g_free(tmpstring);
 	DEBUG_MSG("change_dir, after gfree(tmpstring) \n");
 	return returncode;
-};
+}
 
 /* gchar *strip_any_whitespace(gchar *string)
  * strips any double chars defined by isspace() from the string, 
@@ -386,7 +386,7 @@ gchar *trunc_on_char(gchar * string, gchar which_char)
 		tmpchar++;
 	}
 	return string;
-};
+}
 
 /* gchar *strip_common_path(char *image_fn, char *html_fn)
  * returns a newly allocated string containing the the to_filename
@@ -548,16 +548,18 @@ gchar *create_full_path(gchar * filename, gchar *basedir)
 	gchar *absolute_filename;
 	gchar *tmpcdir;
 
-	DEBUG_MSG("create_full_path, filename=%s, basedir=%p\n", filename, basedir);
+	DEBUG_MSG("create_full_path, filename=%s, basedir=%s\n", filename, basedir);
 	if (g_path_is_absolute(filename)) {
 		absolute_filename = g_strdup(filename);
 	} else {
 		if (basedir) {
-			tmpcdir = g_dirname(basedir);
+			tmpcdir = ending_slash(basedir);
 		} else {
-			tmpcdir = g_get_current_dir();
+			gchar *curdir = g_get_current_dir();
+			tmpcdir = ending_slash(curdir);
+			g_free(curdir);
 		}
-		absolute_filename = g_strconcat(tmpcdir, DIRSTR, filename, NULL);
+		absolute_filename = g_strconcat(tmpcdir, filename, NULL);
 		g_free(tmpcdir);
 	}
 	absolute_filename = most_efficient_filename(absolute_filename);
@@ -595,7 +597,7 @@ gint file_is_dir(gchar * filename) {
 /* gint file_exists_and_readable(gchar * filename)
  * returns 1 if the file pointed to by filename does exist
  * and is readable for the current process */
-gint file_exists_and_readable(gchar * filename)
+gint file_exists_and_readable(const gchar * filename)
 {
 	struct stat naamstat;
 
@@ -603,8 +605,8 @@ gint file_exists_and_readable(gchar * filename)
 	g_assert(filename);
 #endif
 
-	if (strlen(filename) < 2) {
-		DEBUG_MSG("file_exists_and_readable, strlen(filename) < 2!!!!\n");
+	if (!filename || strlen(filename) < 2) {
+		DEBUG_MSG("file_exists_and_readable, strlen(filename) < 2 or no filename!!!!\n");
 		return 0;
 	}
 	DEBUG_MSG("file_exists_and_readable, filename(%p)=\"%s\", strlen(filename)=%d\n", filename, filename, strlen(filename));
@@ -622,6 +624,22 @@ gint file_exists_and_readable(gchar * filename)
 #endif
 }
 
+gchar *return_first_existing_filename(const gchar *filename, ...) {
+	va_list args;
+	gchar *retval=NULL;
+
+	va_start(args, filename);
+	while (filename) {
+		if (file_exists_and_readable(filename)) {
+			retval = filename;
+			break;
+		}
+		filename = va_arg(args, gchar*);
+	}
+	va_end(args);
+	return retval;
+}
+
 /*  gboolean filename_test_extensions(gchar **extensions, gchar *filename)
  *  give a NULL terminated array of strings with all extensions
  *  returns TRUE if the filename has one of those extensions
