@@ -48,7 +48,8 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 gchar *create_full_path(gchar * filename, gchar *basedir);
 gchar *ending_slash(const gchar *dirname);
 gint file_is_dir(gchar * filename);
-gint file_exists_and_readable(gchar * filename);
+gint file_exists_and_readable(const gchar * filename);
+gchar *return_first_existing_filename(const gchar * filename, ...);
 gboolean filename_test_extensions(gchar **extensions, gchar *filename);
 gchar *bf_str_repeat(const gchar * str, gint number_of);
 gint get_int_from_string(gchar *string);
