@@ -23,6 +23,11 @@
 #include <stdlib.h> /* getopt() exit() and abort() on Solaris */
 
 #include "bluefish.h"
+
+#ifdef ENABLE_NLS
+#include <locale.h>
+#endif
+
 #include "document.h" /*  */
 #include "gui.h" /* gui_create_main() */
 #include "rcfile.h" /* rcfile_parse_main() */
@@ -67,7 +72,7 @@ static gint parse_commandline(int argc, char **argv
 		case 'h':
 		case '?':
 			g_print(CURRENT_VERSION_NAME);
-			g_print("\nUsage: %s [options] [filename]\n", argv[0]);
+			g_print(_("\nUsage: %s [options] [filename]\n"), argv[0]);
 			g_print(_("\nCurrently accepted options are:\n"));
 			g_print(_("-s           skip root check\n"));
 			g_print(_("-v           current version\n"));
@@ -102,6 +107,14 @@ int main(int argc, char *argv[])
 #ifndef NOSPLASH
 	GtkWidget *splash_window;
 #endif /* #ifndef NOSPLASH */
+
+#ifdef ENABLE_NLS                                                               
+	setlocale(LC_ALL,"");                                                   
+	bindtextdomain(PACKAGE,LOCALEDIR);
+	DEBUG_MSG("set bindtextdomain for %s to %s\n",PACKAGE,LOCALEDIR);
+	bind_textdomain_codeset(PACKAGE, "UTF-8");
+	textdomain(PACKAGE);                                                    
+#endif    
 	gtk_init(&argc, &argv);
 	
 	main_v = g_new0(Tmain, 1);
