@@ -26,8 +26,9 @@
 #include <string.h> /* strchr() */
 #include <regex.h> 				/* regcomp() */
 #include <stdlib.h> /* system() */
+#include <time.h> /* ctime_r() */
 
-/* #define DEBUG */
+/* #define DEBUG*/
 
 #include "bluefish.h"
 #include "document.h"
@@ -44,6 +45,16 @@
 #include "snr2.h" /* snr2_run_extern_replace */
 #include "filebrowser.h"
 
+typedef struct {
+	Tdocument *last_activated_doc;
+} Tlocals;
+
+/******************************/
+/* global vars for this module */
+/******************************/
+static Tlocals locals = {NULL};
+
+
 void add_filename_to_history(gchar *filename) {
 	gchar *dirname;
 
@@ -78,9 +89,12 @@ gint documentlist_return_index_from_filename(gchar *filename) {
 }
 
 void doc_update_highlighting(GtkWidget *wid, gpointer data) {
+	if (!main_v->current_document) return;
+	DEBUG_MSG("doc_update_highlighting, curdoc=%p, highlightstate=%d\n", main_v->current_document, main_v->current_document->highlightstate);
 	if (main_v->current_document->highlightstate == 0) {
 		setup_toggle_item(gtk_item_factory_from_widget(main_v->menubar),
 			  N_("/Document/Highlight syntax"), TRUE);
+		DEBUG_MSG("doc_update_highlighting, calling doc_toggle_highlighting_cb\n");
 		doc_toggle_highlighting_cb(NULL, 0, NULL);
 	} else {
 		doc_highlight_full(main_v->current_document);
@@ -334,13 +348,19 @@ inline static void doc_update_mtime(Tdocument *doc) {
 }
 /* returns 1 if the time didn't change, returns 0 
 if the file is modified by another process, returns
-1 if there was no previous mtime information available */
-static gint doc_check_mtime(Tdocument *doc) {
+1 if there was no previous mtime information available 
+if newtime is not NULL, it will be filled with the new mtime from the file IF IT WAS CHANGED!!!
+leave NULL if you do not need this information, if the file is not changed, this field will not be set!!
+*/
+static gint doc_check_mtime(Tdocument *doc, time_t *newtime) {
 	if (doc->filename && 0 != doc->mtime) {
 		struct stat statbuf;
 		if (stat(doc->filename, &statbuf) == 0) {
 			if (doc->mtime < statbuf.st_mtime) {
 				DEBUG_MSG("doc_check_mtime, doc->mtime=%d, statbuf.st_mtime=%d\n", (int)doc->mtime, (int)statbuf.st_mtime);
+				if (newtime) {
+					*newtime = statbuf.st_mtime;
+				}
 				return 0;
 			}
 		}
@@ -461,6 +481,11 @@ gboolean doc_get_selection(Tdocument *doc, gint *start, gint *end) {
 	if (*start == *end) {
 		return FALSE;
 	}
+	if (*start > *end) {
+		gint tmp = *start;
+		*start = *end;
+		*end = tmp;
+	}
 	return TRUE;
 }
 
@@ -578,7 +603,7 @@ void doc_insert_two_strings(Tdocument *doc, const gchar *before_str, const gchar
 			gtk_text_iter_backward_chars(&itinsert, strlen(after_str));
 			gtk_text_buffer_place_cursor(doc->buffer, &itinsert);
 		}
-	} else {
+	} else { /* there is a selection */
 		GtkTextMark *marktoresetto;
 		GtkTextIter firstiter;
 		if (gtk_text_iter_compare(&itinsert,&itselect) < 0) {
@@ -655,7 +680,7 @@ gboolean doc_file_to_textbox(Tdocument * doc, gchar * filename, gboolean enable_
 		cursor_offset = gtk_text_iter_get_offset(&iter);
 	}
 	{
-		gchar chunk[STARTING_BUFFER_SIZE+1];
+/*		gchar chunk[STARTING_BUFFER_SIZE+1];
 		gint buffer_size = STARTING_BUFFER_SIZE + 1;
 		gchar *buffer = g_malloc(buffer_size * sizeof(gchar));
 		buffer[0] = '\0';
@@ -663,7 +688,29 @@ gboolean doc_file_to_textbox(Tdocument * doc, gchar * filename, gboolean enable_
 			strcat(buffer, chunk);
 			buffer_size += STARTING_BUFFER_SIZE;
 			buffer = g_realloc(buffer, buffer_size);
+			DEBUG_MSG("doc_file_to_textbox, buffer_size=%d, strlen(buffer)=%d\n",buffer_size, strlen(buffer));
 		}
+		DEBUG_MSG("doc_file_to_textbox, final buffer_size=%d, strlen(buffer)=%d\n", buffer_size, strlen(buffer));*/
+		gchar chunk[STARTING_BUFFER_SIZE];
+		gint buffer_size = STARTING_BUFFER_SIZE;
+		size_t size = STARTING_BUFFER_SIZE;
+		gchar *buffer = g_malloc(1+(buffer_size * sizeof(gchar)));
+		while (TRUE) {
+			size = fread(chunk,sizeof(gchar),STARTING_BUFFER_SIZE,fd);
+			memcpy(&buffer[buffer_size-STARTING_BUFFER_SIZE], chunk, STARTING_BUFFER_SIZE);
+			DEBUG_MSG("doc_file_to_textbox, copy %d bytes to position %d, size=%d\n",STARTING_BUFFER_SIZE, buffer_size-STARTING_BUFFER_SIZE, size);
+			if (size == STARTING_BUFFER_SIZE) {
+				buffer_size += STARTING_BUFFER_SIZE;
+				buffer = g_realloc(buffer, 1+(buffer_size * sizeof(gchar)));
+			} else {
+				DEBUG_MSG("size=%d, terminating buffer at %d\n",size,buffer_size-STARTING_BUFFER_SIZE+size);
+				buffer[buffer_size-STARTING_BUFFER_SIZE+size] = '\0';
+				break;
+			}
+		}
+		DEBUG_MSG("doc_file_to_textbox, final buffer_size=%d, strlen(buffer)=%d\n", buffer_size, strlen(buffer));
+		fclose(fd);
+		
 		/* the first try is if the encoding is set in the file */
 		{
 			regex_t preg;
@@ -773,16 +820,16 @@ gboolean doc_file_to_textbox(Tdocument * doc, gchar * filename, gboolean enable_
 			g_free(buffer);
 		}
 	}
-	fclose(fd);
-	doc->need_highlighting=TRUE;
-	if (!delay_highlighting) {
+	if (doc->highlightstate) {
+		doc->need_highlighting=TRUE;
+		DEBUG_MSG("doc_file_to_textbox, highlightstate=%d, need_highlighting=%d, delay_highlighting=%d\n",doc->highlightstate,doc->need_highlighting,delay_highlighting);
+		if (!delay_highlighting) {
 #ifdef DEBUG
-		g_print("doc_file_to_textbox, doc->hlset=%p\n", doc->hl);
-		if (doc->hl) {
-			g_print("doc_file_to_textbox, doc->hlset->highlightlist=%p\n", doc->hl->highlightlist);
-		}
+			g_print("doc_file_to_textbox, doc->hlset=%p\n", doc->hl);
+			if (doc->hl) {
+				g_print("doc_file_to_textbox, doc->hlset->highlightlist=%p\n", doc->hl->highlightlist);
+			}
 #endif
-		if (main_v->props.cont_highlight_update) {
 			doc_highlight_full(doc);
 		}
 	}
@@ -869,16 +916,28 @@ static void doc_buffer_insert_text_lcb(GtkTextBuffer *textbuffer,GtkTextIter * i
 }
 
 static void doc_buffer_insert_text_after_lcb(GtkTextBuffer *textbuffer,GtkTextIter * iter,gchar * string,gint len, Tdocument * doc) {
+	gboolean do_highlighting = FALSE;
 	/* highlighting stuff */
 	DEBUG_MSG("doc_buffer_insert_text_after_lcb, started\n");
-	if (string && doc->hl->update_chars) {
-		gint i=0;
-		while (string[i] != '\0') {
-			if (strchr(doc->hl->update_chars, string[i])) {
+	if (doc->highlightstate && string && doc->hl) {
+		if (strlen(doc->hl->update_chars)==0 ) {
+			do_highlighting = TRUE;
+		} else {
+			gint i=0;
+			while (string[i] != '\0') {
+				if (strchr(doc->hl->update_chars, string[i])) {
+					do_highlighting = TRUE;
+					break;
+				}
+				i++;
+			}
+		}
+		if (do_highlighting) {
+			if (main_v->props.cont_highlight_full) {
+				doc_highlight_full(doc);
+			} else {
 				doc_highlight_line(doc);
-				break;
 			}
-			i++;
 		}
 	}
 
@@ -914,17 +973,29 @@ static void doc_buffer_insert_text_after_lcb(GtkTextBuffer *textbuffer,GtkTextIt
 
 static void doc_buffer_delete_range_lcb(GtkTextBuffer *textbuffer,GtkTextIter * itstart,GtkTextIter * itend, Tdocument * doc) {
 	gchar *string;
-	gint i=0;
+	gboolean do_highlighting=FALSE;
 	string = gtk_text_buffer_get_text(doc->buffer, itstart, itend, FALSE);
 	if (string) {
 		/* highlighting stuff */
-		if (doc->hl->update_chars) {
-			while (string[i] != '\0') {
-				if (strchr(doc->hl->update_chars, string[i])) {
+		if (doc->highlightstate && string && doc->hl) {
+			if (strlen(doc->hl->update_chars)==0 ) {
+				do_highlighting = TRUE;
+			} else {
+				gint i=0;
+				while (string[i] != '\0') {
+					if (strchr(doc->hl->update_chars, string[i])) {
+						do_highlighting = TRUE;
+						break;
+					}
+					i++;
+				}
+			}
+			if (do_highlighting) {
+				if (main_v->props.cont_highlight_full) {
+					doc_highlight_full(doc);
+				} else {
 					doc_highlight_line(doc);
-					break;
 				}
-				i++;
 			}
 		}
 		/* undo_redo stuff */
@@ -947,7 +1018,6 @@ static void doc_buffer_delete_range_lcb(GtkTextBuffer *textbuffer,GtkTextIter *
 			}
 			doc_unre_add(doc, string, start, end, UndoDelete);
 		}
-
 		g_free(string);
 	}
 	doc_set_modified(doc, 1);
@@ -1055,10 +1125,10 @@ gint doc_textbox_to_file(Tdocument * doc, gchar * filename) {
 		gchar *tmp;
 		Tsearch_result res = doc_search_run_extern("<meta[ \t\n]name[ \t\n]*=[ \t\n]*\"generator\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*bluefish[^\"]*\"[ \t\n]*>",1,0);
 		if (res.end > 0) {
-			snr2_run_extern_replace("<meta[ \t\n]name[ \t\n]*=[ \t\n]*\"generator\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*\"[ \t\n]*>",0,1,0,"<meta name=\"generator\" content=\"Bluefish, see http://bluefish.openoffice.nl/\">");
+			snr2_run_extern_replace("<meta[ \t\n]name[ \t\n]*=[ \t\n]*\"generator\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*\"[ \t\n]*>",0,1,0,"<meta name=\"generator\" content=\"Bluefish, see http://bluefish.openoffice.nl/\">", FALSE);
 		}
 		tmp = g_strconcat("<meta name=\"author\" content=\"",realname,"\">",NULL);
-		snr2_run_extern_replace("<meta[ \t\n]name[ \t\n]*=[ \t\n]*\"author\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*\"[ \t\n]*>",0,1,0,tmp);
+		snr2_run_extern_replace("<meta[ \t\n]name[ \t\n]*=[ \t\n]*\"author\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*\"[ \t\n]*>",0,1,0,tmp,FALSE);
 		g_free(tmp);
 	}
 
@@ -1123,25 +1193,39 @@ void doc_destroy(Tdocument * doc, gboolean delay_activation)
 	if (doc->filename) {
 		add_to_recent_list(doc->filename, 1);
 	}
-
-	if (delay_activation) {
-		gui_notebook_unbind_signals();
-	}
-
-	gtk_notebook_remove_page(GTK_NOTEBOOK(main_v->notebook),
-							 gtk_notebook_page_num(GTK_NOTEBOOK(main_v->notebook),doc->view->parent));
-
+	gui_notebook_unbind_signals();
+	/* to make this go really quick, we first only destroy the notebook page and run flush_queue(), 
+	after the document is gone from the GUI we complete the destroy, to destroy only the notebook
+	page we ref+ the scrolthingie, remove the page, and unref it again */
+	g_object_ref(doc->view->parent);
+	/* now we remove the document from the document list */
 	if (g_list_length(main_v->documentlist) > 1) {
 		main_v->documentlist = g_list_remove(main_v->documentlist, doc);
+		DEBUG_MSG("removed %p from documentlist, list length=%d\n",doc, g_list_length(main_v->documentlist));
 	} else {
 		main_v->documentlist = g_list_remove(main_v->documentlist, doc);
-		DEBUG_MSG("doc_destroy, last document removed from documentlist\n");
+		DEBUG_MSG("doc_destroy, last document removed from documentlist, documentlist=%p\n", main_v->documentlist);
 		g_list_free(main_v->documentlist);
 		DEBUG_MSG("doc_destroy, freed documentlist\n");
 		main_v->documentlist = NULL;
 		DEBUG_MSG("doc_destroy, documentlist = NULL\n");
 	}
 	DEBUG_MSG("doc_destroy, g_list_length(documentlist)=%d\n",g_list_length(main_v->documentlist));
+	if (main_v->current_document == doc) {
+		main_v->current_document = NULL;
+	}
+	/* then we remove the page from the notebook */
+	gtk_notebook_remove_page(GTK_NOTEBOOK(main_v->notebook),
+							 gtk_notebook_page_num(GTK_NOTEBOOK(main_v->notebook),doc->view->parent));
+/*	flush_queue();*/
+	if (!delay_activation) {
+		notebook_changed(-1);
+	}
+	gui_notebook_bind_signals();
+
+	/* now we really start to destroy the document */
+	g_object_unref(doc->view->parent);
+
 	if (doc->filename) {
 		g_free(doc->filename);
 	}
@@ -1151,11 +1235,8 @@ void doc_destroy(Tdocument * doc, gboolean delay_activation)
 	
 	doc_unre_destroy(doc);
 	g_free(doc);
-	if (!delay_activation) {
-		notebook_changed(-1);
-	} else {
-		gui_notebook_bind_signals();
-	}
+
+	DEBUG_MSG("doc_destroy, finished for %p\n", doc);
 }
 
 /* gint doc_save(Tdocument * doc, gint do_save_as, gint do_move)
@@ -1235,13 +1316,16 @@ gint doc_save(Tdocument * doc, gint do_save_as, gint do_move)
 		doc->filename = oldfilename;
 		return -4;
 	}
-	
-	if (doc_check_mtime(doc) == 0) {
-		gchar *tmpstr;
+	{
+	time_t newtime;
+	if (doc_check_mtime(doc,&newtime) == 0) {
+		gchar *tmpstr, oldtimestr[128], newtimestr[128];/* according to 'man ctime_r' this should be at least 26, so 128 should do ;-)*/
 		gint retval;
 		gchar *options[] = {N_("Overwrite"), N_("Cancel"), NULL};
 
-		tmpstr = g_strdup_printf(_("File %s\nis modified by another process, overwrite?"), doc->filename);
+		ctime_r(&newtime,newtimestr);
+		ctime_r(&doc->mtime,oldtimestr);
+		tmpstr = g_strdup_printf(_("File %s\nis modified by another process\nnew modification time is %s\nold modification time is %s\noverwrite?"), doc->filename,newtimestr,oldtimestr);
 		retval = multi_button_dialog(_("Bluefish: Warning, file is modified"), 0, tmpstr, options);
 		g_free(tmpstr);
 		if (retval == 1) {
@@ -1251,6 +1335,7 @@ gint doc_save(Tdocument * doc, gint do_save_as, gint do_move)
 			return -5;
 		}
 	}
+	}
 	
 	DEBUG_MSG("doc_save, returned file %s\n", doc->filename);
 	if (do_save_as && oldfilename && main_v->props.link_management) {
@@ -1356,12 +1441,12 @@ gint doc_close(Tdocument * doc, gint warn_only)
 			break;
 		}
 	} else {
-		DEBUG_MSG("doc_close, closing doc=%p\n", doc);
 		if (!warn_only) {
+			DEBUG_MSG("doc_close, starting doc_destroy for doc=%p\n", doc);
 			doc_destroy(doc, FALSE);
 		}
 	}
-
+	DEBUG_MSG("doc_close, finished\n");
 /*	notebook_changed();*/
 	return 1;
 }
@@ -1370,6 +1455,58 @@ static void doc_close_but_clicked_lcb(GtkWidget *wid, gpointer data) {
 	doc_close(data, 0);
 }
 
+/* contributed by Oskar Swida <swida@aragorn.pb.bialystok.pl>, with help from the gedit source */
+static gboolean doc_textview_expose_event_lcb(GtkWidget * widget, GdkEventExpose * event, gpointer data) {
+	GtkTextView *view = (GtkTextView*)widget;
+	GdkRectangle rect;
+	GdkWindow *win;
+	GtkTextIter l_start,l_end,it;
+	gint l_top1,l_top2;
+	PangoLayout *l;
+	gchar *pomstr;
+	gint numlines,w,i;
+
+	win = gtk_text_view_get_window(view,GTK_TEXT_WINDOW_LEFT);
+	if (win!=event->window) return FALSE;
+
+	gtk_text_view_get_visible_rect(view,&rect);
+	gtk_text_view_get_line_at_y(view,&l_start,rect.y,&l_top1);
+	gtk_text_view_get_line_at_y(view,&l_end,rect.y+rect.height,&l_top2);
+	l = gtk_widget_create_pango_layout(widget,"");
+
+	numlines = gtk_text_buffer_get_line_count(gtk_text_view_get_buffer(view));
+	pomstr = g_strdup_printf("%d",MAX(99,numlines));
+	pango_layout_set_text(l,pomstr,-1);
+	g_free(pomstr);
+	pango_layout_get_pixel_size(l,&w,NULL);
+	gtk_text_view_set_border_window_size(view,GTK_TEXT_WINDOW_LEFT,w+4);   
+	it = l_start;
+	for(i=gtk_text_iter_get_line(&l_start);i<=gtk_text_iter_get_line(&l_end);i++) {
+		gtk_text_iter_set_line(&it,i);
+		gtk_text_view_get_line_yrange(view,&it,&w,NULL);
+      
+		gtk_text_view_buffer_to_window_coords(view,GTK_TEXT_WINDOW_LEFT,0,w,NULL,&w);
+		pomstr = g_strdup_printf("%d",i+1);
+		pango_layout_set_text(l,pomstr,-1);
+      
+		gtk_paint_layout(widget->style,win,GTK_WIDGET_STATE(widget),FALSE,NULL,widget,NULL,2,w,l);
+      g_free(pomstr);
+	}
+	g_object_unref(G_OBJECT(l));
+	return TRUE;
+}
+
+void document_set_line_numbers(Tdocument *doc, gboolean value) {
+	if (value) {
+		gtk_text_view_set_left_margin(GTK_TEXT_VIEW(doc->view),2);
+		gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(doc->view),GTK_TEXT_WINDOW_LEFT,20);
+		g_signal_connect(G_OBJECT(doc->view),"expose-event",G_CALLBACK(doc_textview_expose_event_lcb),NULL);
+	} else {
+		gtk_text_view_set_left_margin(GTK_TEXT_VIEW(doc->view),0);
+		gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(doc->view),GTK_TEXT_WINDOW_LEFT,0);
+	}
+}
+
 Tdocument *doc_new(gboolean delay_activate) {
 	GtkWidget *scroll;
 	Tdocument *newdoc = g_new0(Tdocument, 1);
@@ -1386,6 +1523,9 @@ Tdocument *doc_new(gboolean delay_activate) {
 											(scroll), GTK_SHADOW_IN);
 	gtk_container_add(GTK_CONTAINER(scroll), newdoc->view);
 
+	newdoc->linenumberstate = main_v->props.view_line_numbers;
+	document_set_line_numbers(newdoc, newdoc->linenumberstate);
+
 	newdoc->tab_label = gtk_label_new(NULL);
 	GTK_WIDGET_UNSET_FLAGS(newdoc->tab_label, GTK_CAN_FOCUS);
 	if (strlen(main_v->props.tab_font_string)) {
@@ -1445,6 +1585,7 @@ Tdocument *doc_new(gboolean delay_activate) {
 		gtk_notebook_append_page_menu(GTK_NOTEBOOK(main_v->notebook), scroll ,hbox, newdoc->tab_menu);
 	}
 	newdoc->highlightstate = main_v->props.defaulthighlight;
+	DEBUG_MSG("doc_new, need_highlighting=%d, highlightstate=%d\n", newdoc->need_highlighting, newdoc->highlightstate);
 	if (!delay_activate) {
 		DEBUG_MSG("doc_new, set notebook page to %d\n", g_list_length(main_v->documentlist) - 1);
 		gtk_notebook_set_page(GTK_NOTEBOOK(main_v->notebook),g_list_length(main_v->documentlist) - 1);
@@ -1487,11 +1628,13 @@ gboolean doc_new_with_file(gchar * filename, gboolean delay_activate) {
 	Tdocument *doc;
 	
 	if ((filename == NULL) || (!file_exists_and_readable(filename))) {
+		DEBUG_MSG("doc_new_with_file, file %s !file_exists or readable\n", filename);
 		return FALSE;
 	}
 	if (!main_v->props.allow_multi_instances) {
 		gint index = documentlist_return_index_from_filename(filename);
 		if (index != -1) {
+			DEBUG_MSG("doc_new_with_file, %s is already open at index=%d\n",filename,index);
 			if (!delay_activate) {
 				switch_to_document_by_index(index);
 			}
@@ -1565,12 +1708,25 @@ void doc_reload(Tdocument *doc) {
 }
 
 void doc_activate(Tdocument *doc) {
-	if (doc_check_mtime(doc) == 0) {
-		gchar *tmpstr;
+	time_t newtime;
+#ifdef DEBUG
+	if (!doc) {
+		DEBUG_MSG("doc_activate, doc=NULL!!! ABORTING!!\n");
+		exit(44);
+	}
+#endif
+	if (doc == locals.last_activated_doc) {
+		return;
+	}
+	locals.last_activated_doc = doc;
+	if (doc_check_mtime(doc,&newtime) == 0) {
+		gchar *tmpstr, oldtimestr[128], newtimestr[128];/* according to 'man ctime_r' this should be at least 26, so 128 should do ;-)*/
 		gint retval;
 		gchar *options[] = {N_("Reload"), N_("Ignore"), NULL};
 
-		tmpstr = g_strdup_printf(_("File %s\nis modified by another process"), doc->filename);
+		ctime_r(&newtime,newtimestr);
+		ctime_r(&doc->mtime,oldtimestr);
+		tmpstr = g_strdup_printf(_("File %s\nis modified by another process\nnew modification time is %s\nold modification time is %s"), doc->filename,newtimestr,oldtimestr);
 		retval = multi_button_dialog(_("Bluefish: Warning, file is modified"), 0, tmpstr, options);
 		g_free(tmpstr);
 		if (retval == 1) {
@@ -1580,7 +1736,7 @@ void doc_activate(Tdocument *doc) {
 		}
 	}
 	DEBUG_MSG("doc_activate, calling gui_set_widgets\n");
-	gui_set_widgets(doc_has_undo_list(doc), doc_has_redo_list(doc), doc->wrapstate, doc->highlightstate, doc->hl, doc->encoding);
+	gui_set_widgets(doc_has_undo_list(doc), doc_has_redo_list(doc), doc->wrapstate, doc->highlightstate, doc->hl, doc->encoding, doc->linenumberstate);
 
 	/* if highlighting is needed for this document do this now !! */
 	if (doc->need_highlighting && doc->highlightstate) {
@@ -1590,7 +1746,6 @@ void doc_activate(Tdocument *doc) {
 
 	doc_scroll_to_cursor(doc);
 	gtk_widget_grab_focus(GTK_WIDGET(doc->view));
-	flush_queue();
 	if (doc->filename) {
 		gchar *dir1 = g_path_get_dirname(doc->filename);
 		gchar *dir2 = ending_slash(dir1);
@@ -1604,7 +1759,8 @@ void doc_activate(Tdocument *doc) {
 /**************************************************************************/
 /* the start of the callback functions for the menu, acting on a document */
 /**************************************************************************/
-
+#ifdef EXTERNAL_GREP
+#ifdef EXTERNAL_FIND
 typedef struct {
 	GList *filenames_to_return;
 	GtkWidget *win;
@@ -1696,7 +1852,7 @@ static void files_advanced_win(Tfiles_advanced *tfs) {
 	/* filename part */
 	/* curdir should get a value */
 	{
-		GtkWidget *but = bf_stock_button(_("Browse"), G_CALLBACK(files_advanced_win_select_basedir_lcb), tfs);
+		GtkWidget *but = bf_gtkstock_button(GTK_STOCK_OPEN, G_CALLBACK(files_advanced_win_select_basedir_lcb), tfs);
 		hbox = gtk_hbox_new(FALSE,3);
 		gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Basedir")), FALSE, FALSE, 2);
 		tfs->basedir = boxed_entry_with_text(curdir, 255, hbox);
@@ -1742,8 +1898,8 @@ static void files_advanced_win(Tfiles_advanced *tfs) {
 	gtk_box_pack_start(GTK_BOX(hbox),but , FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show_all(GTK_WIDGET(tfs->win));
-	gtk_grab_add(GTK_WIDGET(tfs->win));
-	gtk_widget_realize(GTK_WIDGET(tfs->win));
+/*	gtk_grab_add(GTK_WIDGET(tfs->win));
+	gtk_widget_realize(GTK_WIDGET(tfs->win));*/
 	gtk_window_set_transient_for(GTK_WINDOW(tfs->win), GTK_WINDOW(main_v->main_window));
 }
 
@@ -1754,8 +1910,9 @@ GList *return_files_advanced() {
 	gtk_main();
 	return tfs.filenames_to_return;
 }	
+#endif /* EXTERNAL_FIND */
+#endif /* EXTERNAL_GREP */
 
-	
 
 void file_save_cb(GtkWidget * widget, gpointer data) {
 	doc_save(main_v->current_document, 0, 0);
@@ -1773,13 +1930,24 @@ void file_move_to_cb(GtkWidget * widget, gpointer data) {
 void file_open_cb(GtkWidget * widget, gpointer data) {
 	GList *tmplist;
 	if (GPOINTER_TO_INT(data) == 1) {
+#ifdef EXTERNAL_GREP
+#ifdef EXTERNAL_FIND
 		tmplist = return_files_advanced();
+#endif
+#endif
 	} else {
 		tmplist = return_files(NULL);
 	}
 	if (!tmplist) {
 		return;
 	}
+	{
+		gint len = g_list_length(tmplist);
+		gchar *message = g_strdup_printf(_("Loading %d file(s)..."), len);
+		statusbar_message(message,2000+len*50);
+		g_free(message);
+		flush_queue();
+	}
 	docs_new_from_files(tmplist);
 	free_stringlist(tmplist);
 }
@@ -1827,7 +1995,7 @@ void file_close_all_cb(GtkWidget * widget, gpointer data)
 
 	/* first a warning loop */
 	if (test_docs_modified(NULL)) {
-		gchar *options[] = {N_("save all"), N_("close all"), N_("choose per file"), N_("cancel"), NULL};
+		gchar *options[] = {_("save all"), _("close all"), _("choose per file"), _("cancel"), NULL};
 		retval =	multi_button_dialog(_("Bluefish: Warning, some file(s) are modified!"), 3, _("Some file(s) are modified\nplease choose your action"), options);
 		if (retval == 3) {
 			DEBUG_MSG("file_close_all_cb, cancel clicked, returning 0\n");
@@ -1921,8 +2089,9 @@ void edit_select_all_cb(GtkWidget * widget, gpointer data) {
 }
 
 void doc_toggle_highlighting_cb(gpointer callback_data,guint action,GtkWidget *widget) {
-	DEBUG_MSG("doc_toggle_highlighting_cb, started\n");
+
 	main_v->current_document->highlightstate = 1 - main_v->current_document->highlightstate;
+	DEBUG_MSG("doc_toggle_highlighting_cb, started, highlightstate now is %d\n", main_v->current_document->highlightstate);
 	if (main_v->current_document->highlightstate == 0) {
 		doc_remove_highlighting(main_v->current_document);
 	} else {
@@ -1935,7 +2104,10 @@ void doc_toggle_wrap_cb(gpointer callback_data,guint action,GtkWidget *widget) {
 	doc_set_wrap(main_v->current_document);
 }
 
-
+void doc_toggle_linenumbers_cb(gpointer callback_data,guint action,GtkWidget *widget) {
+	main_v->current_document->linenumberstate = 1 - main_v->current_document->linenumberstate;
+	document_set_line_numbers(main_v->current_document, main_v->current_document->linenumberstate);
+}
 
 /* callback_action: 1 only ascii, 2 only iso, 3 both
  */
