@@ -120,6 +120,7 @@ void setup_toggle_item(GtkItemFactory * ifactory, gchar * path, gint state)
 		DEBUG_MSG("warning, this menuitem (%s) does NOT exist!", path);
 		return;
 	}
+	DEBUG_MSG("setting path %s to %d\n",path,state);
 	if ((GTK_CHECK_MENU_ITEM(toggle)->active) && state == 0) {
 		GTK_CHECK_MENU_ITEM(toggle)->active = state;
 		return;
@@ -504,7 +505,7 @@ GtkWidget *bf_stock_button(const gchar * Text, GCallback func, gpointer func_dat
 	g_signal_connect(G_OBJECT(button), "clicked", func, func_data);
 	DEBUG_MSG("bf_stock_button, func_data=%p\n", func_data);
 	return button;
-};
+}
 
 GtkWidget *bf_gtkstock_button(const gchar * stock_id, GCallback func, gpointer func_data)
 {
@@ -515,7 +516,7 @@ GtkWidget *bf_gtkstock_button(const gchar * stock_id, GCallback func, gpointer f
 	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 	g_signal_connect(G_OBJECT(button), "clicked", func, func_data);
 	return button;
-};
+}
 
 
 /*
@@ -530,7 +531,7 @@ GtkWidget *bf_gtkstock_button(const gchar * stock_id, GCallback func, gpointer f
  */
 GtkWidget *bf_stock_ok_button(GtkSignalFunc func, gpointer func_data) {
 	return bf_gtkstock_button(GTK_STOCK_OK, func, func_data);
-};
+}
 
 /*
  * Function: bf_stock_cancel_button
@@ -545,7 +546,7 @@ GtkWidget *bf_stock_ok_button(GtkSignalFunc func, gpointer func_data) {
 GtkWidget *bf_stock_cancel_button(GCallback func, gpointer func_data)
 {
 	return bf_gtkstock_button(GTK_STOCK_CANCEL, func, func_data);
-};
+}
 
 /*
  * Function: error_dialog
@@ -757,8 +758,8 @@ static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
 	if (file_is_dir(selected_file)) {
 		DEBUG_MSG("fs_ok_clicked_lcb,file_is_dir said %s is a dir!!!!\n", selected_file);
 		if (fileselect->select_dir) {
-			fileselect->filename_to_return = selected_file;
-			g_free(selected_file);	
+			fileselect->filename_to_return = g_path_get_dirname(selected_file);
+			g_free(selected_file);
 			gtk_main_quit();
 			window_destroy(GTK_WIDGET(fileselect->fs));
 			return;
@@ -778,14 +779,20 @@ static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
 		gchar **filenames, **orig;
 		/* multiple files allowed --> scan trough the list for selections */
 		orig = filenames = gtk_file_selection_get_selections(GTK_FILE_SELECTION(fileselect->fs));
-		while (*filenames) {
-			fileselect->filenames_to_return = g_list_append(fileselect->filenames_to_return, g_strdup(*filenames));
-			filenames++;
+		if (filenames) {
+			while (*filenames) {
+				fileselect->filenames_to_return = g_list_append(fileselect->filenames_to_return, g_strdup(*filenames));
+				filenames++;
+			}
+			g_strfreev(orig);
 		}
-		g_strfreev(orig);
 	} else {
 		/* NO multiple files allowed --> return just one file */
-		fileselect->filename_to_return = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs)));
+		if (fileselect->select_dir) {
+			fileselect->filename_to_return = g_path_get_dirname(selected_file);
+		} else {
+				fileselect->filename_to_return = g_strdup(selected_file);
+		}
 	}
 	if ((fileselect->multipleselect 
 			&& (g_list_length(fileselect->filenames_to_return) < 1)) 
@@ -833,7 +840,8 @@ static void fileselectwin(gchar *setfile, Tfileselect *fileselect, gchar *title)
 
 	DEBUG_MSG("fileselectwin, started\n");
 	fileselect->fs = gtk_file_selection_new(title);
-	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fileselect->fs));
+	gtk_file_selection_show_fileop_buttons(GTK_FILE_SELECTION(fileselect->fs));
+	/*gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fileselect->fs));*/
 	g_signal_connect(G_OBJECT(fileselect->fs), "destroy", G_CALLBACK(close_modal_window_lcb), fileselect->fs);
 	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fileselect->fs)->cancel_button),
 					   "clicked", G_CALLBACK(close_modal_window_lcb), fileselect->fs);
@@ -849,7 +857,7 @@ static void fileselectwin(gchar *setfile, Tfileselect *fileselect, gchar *title)
 	gtk_window_set_role(GTK_WINDOW(fileselect->fs), "fileselect");
 
 	gtk_widget_show(fileselect->fs);
-	gtk_grab_add(GTK_WIDGET(fileselect->fs));
+/*	gtk_grab_add(GTK_WIDGET(fileselect->fs));*/
 	gtk_widget_realize(GTK_WIDGET(fileselect->fs));
 	gtk_window_set_transient_for(GTK_WINDOW(fileselect->fs), GTK_WINDOW(main_v->main_window));
 }
