@@ -25,6 +25,8 @@
 #include <unistd.h> /* exit() */
 #include <stdlib.h> /* exit() on Solaris */
 
+#include "config.h"
+
 #include "bluefish.h"
 #include "gui.h"
 #include "document.h" /* file_new_cb() */
@@ -45,6 +47,9 @@
 #include "preferences.h" /* open_preferences_cb */
 #include "outputbox.h" /* init_outputbox() */
 
+#ifdef HAVE_LIBASPELL
+#include "bfspell.h"
+#endif /* HAVE_LIBASPELL */
 typedef struct {
 	GtkWidget *main_toolbar_hb;
 	GtkWidget *html_toolbar_hb;
@@ -80,6 +85,8 @@ static Tsplashscreen splashscreen;
 void notebook_changed(gint newpage)
 {
 	gint cur;
+	DEBUG_MSG("notebook_changed, the newpage argument is %d\n", newpage);
+	DEBUG_MSG("notebook_changed, documentlist len=%d\n", g_list_length(main_v->documentlist));
 
 	if (newpage == -1) {
 		/* This one is called when you click on the notebook
@@ -88,14 +95,17 @@ void notebook_changed(gint newpage)
 	} else {
 		cur = newpage;
 	}
-	DEBUG_MSG("notebook_changed, main_v is at %p\n", main_v);
-
+	DEBUG_MSG("notebook_changed, cur now is %d\n", cur);
 	if ((main_v->last_notebook_page == cur) 
 		&& (main_v->current_document != NULL)
 		&& (main_v->current_document == g_list_nth_data(main_v->documentlist, cur))) {
 		DEBUG_MSG("notebook_changed, it didn't change to a new document (cur=%d, current_document=%p)\n", cur, main_v->current_document);
 		return;
 	} else {
+		if (cur >= g_list_length(main_v->documentlist)) {
+			cur = -1;
+		}
+		DEBUG_MSG("after checking documentlist len(%d) cur=%d\n",g_list_length(main_v->documentlist),cur);
 		if (cur == -1) {
 			DEBUG_MSG("notebook_changed, cur=-1, calling file_new_cb()\n");
 			file_new_cb(NULL, NULL);
@@ -108,6 +118,10 @@ void notebook_changed(gint newpage)
 			DEBUG_MSG("notebook_changed, finished, main_v->current_document=%p\n", main_v->current_document);
 			g_assert(main_v->current_document);
 #endif
+			/* now we flush the queue first, so that we don't call doc_activate 
+			on _this_ document if the user has another close click in the queue */
+			flush_queue();
+
 			doc_activate(main_v->current_document);
 		}
 	}
@@ -517,9 +531,11 @@ void make_main_toolbar(GtkWidget *handlebox) {
 							"", new_pixmap(12), G_CALLBACK(open_preferences_cb), NULL);
 /*
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Print..."), "",
-							new_pixmap(015), G_CALLBACK(file_print_cb), NULL);
+							new_pixmap(015), G_CALLBACK(file_print_cb), NULL);*/
+#ifdef HAVE_LIBASPELL
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Spellcheck..."),
-							"", new_pixmap(016), G_CALLBACK(spell_check_cb), NULL);*/
+							"", new_pixmap(104), G_CALLBACK(spell_check_cb), NULL);
+#endif /* HAVE_LIBASPELL */
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL,
 							_("View in browser"), "",
@@ -538,10 +554,11 @@ void gui_set_undo_redo_widgets(gboolean undo, gboolean redo) {
 	gtk_widget_set_sensitive(gtk_item_factory_get_widget(gtk_item_factory_from_widget(main_v->menubar), N_("/Edit/Redo all")), redo);
 }
 
-void gui_set_widgets(gboolean undo, gboolean redo, gboolean wrap, gboolean highlight, Tfiletype *hl, gchar *encoding) {
+void gui_set_widgets(gboolean undo, gboolean redo, gboolean wrap, gboolean highlight, Tfiletype *hl, gchar *encoding, gboolean linenumbers) {
 	gui_set_undo_redo_widgets(undo, redo);
 	setup_toggle_item(gtk_item_factory_from_widget(main_v->menubar),N_("/Document/Highlight syntax"), highlight);
 	setup_toggle_item(gtk_item_factory_from_widget(main_v->menubar),N_("/Document/Wrap"), wrap);
+	setup_toggle_item(gtk_item_factory_from_widget(main_v->menubar),N_("/Document/Line numbers"), linenumbers);
 	menu_current_document_set_toggle_wo_activate(hl, encoding);
 }
 
@@ -585,7 +602,7 @@ static void main_win_on_drag_data_lcb(GtkWidget * widget, GdkDragContext * conte
 
 	/* netscape sends URL's labelled as string */
 	if (info == TARGET_STRING) {
-		gchar *stringdata = g_strndup(data->data, data->length);
+		gchar *stringdata = g_strndup((gchar *)data->data, data->length);
 		if (strchr(stringdata, ':')) {
 			DEBUG_MSG("on_drag_data_cb, TARGET_STRING contains :, so it's probably an URL\n");
 			info = TARGET_URI_LIST;
@@ -665,7 +682,7 @@ void gui_create_main(GList *filenames) {
 	/* first a menubar */
 	menu_create_main(vbox);
 	recent_menu_init();
-	external_menu_init();
+	external_menu_rebuild();
 	encoding_menu_rebuild();
 	menu_outputbox_rebuild();
 
