@@ -22,10 +22,10 @@
  * indent --line-length 100 --k-and-r-style --tab-size 4 -bbo --ignore-newlines highlight.c
  */
 /*#define HL_TIMING
-#define HL_DEBUG
-#define DEBUG*/
+#define HL_DEBUG 
+#define DEBUG */
 
-#ifdef HL_TIMING
+#ifdef HL_TIMING /* some overall profiling information, not per pattern, but per type of code , interesting to bluefish programmers*/
 #include <sys/times.h>
 #include <unistd.h>
 #endif
@@ -36,6 +36,13 @@
 #include <string.h>				/* strerror() */
 #include <stdlib.h>				/* atoi() */
 
+#include "config.h" /* HL_PROFILING might be defined there */
+
+#ifdef HL_PROFILING /* per pattern profiling information, interesting for users making a new pattern */
+#include <sys/times.h>
+#include <unistd.h>
+#endif
+
 #include "bluefish.h"
 #include "bf_lib.h"				/* filename_test_extensions() */
 #include "rcfile.h"				/* array_from_arglist() */
@@ -44,9 +51,9 @@
 #include "document.h" /* doc_get_chars() */
 #include "highlight.h"
 
-
-#define NUM_SUBMATCHES 30 /* should be a multiple of three for pcre_exec(), and at maximum 2/3 of this size can really be used for substrings */
-
+#define MAX_OVECTOR 30 /* should be a multiple of three for pcre_exec(), 
+									and at maximum 2/3 of this size can really be used for substrings */
+#define MIN_OVECTOR 9 /* the minimum size for the ovector */
 typedef struct {
 	pcre *pcre;
 	pcre_extra *pcre_e;
@@ -64,7 +71,28 @@ typedef struct {
 	gint mode;
 	GList *childs;
 	GtkTextTag *tag;
+#ifdef HL_PROFILING
+	gint runcount;
+	gint applycount;
+	struct tms tms1;
+	struct tms tms2;
+	glong total_ms;
+#endif
+	int *ovector;
+	gint ovector_size;
+	gboolean is_match;
 } Tpattern;
+/* 
+proposed improvement (for speedup, but also simplicity):
+	-add the int ovector[MAX_OVECTOR]; and gboolean is_match; fields to Tpattern
+	-use the pcre_fullinfo() function to find the minimum required ovector[] length (remind the 3X!!)
+	and if there are no child-patterns that need a subpattern we use the minimum
+	-the results from a subpattern search are now automatically stored, improving the success ratio
+	for applying/matching*100% as in the profiling information
+	-the Tpatmatch structure is not needed anymore, also the start of applylevel() where this 
+	structure is initialized is now more simple
+*/
+
 
 typedef struct {
 	gchar *filetype;
@@ -78,13 +106,17 @@ typedef struct {
 	GtkTextTagTable *tagtable; /* this one should ultimately move to Tfiletype, so every set would have it's own tagtable, but there is currently no way to switch a document to a new tagtable */
 	GList *highlight_filetypes; /* contains all filetypes that have a highlight pattern */
 	GList *all_highlight_patterns; /* contains Tmetapattern, not Tpattern !! */
+#ifdef HL_PROFILING
+	struct tms tms1; /* start time for profiling info */
+	struct tms tms2; /* stop time for profiling info */
+#endif
 } Thighlight;
 
-typedef struct {
-	int ovector[NUM_SUBMATCHES];
+/*typedef struct {
+	int ovector[MAX_OVECTOR];
 	gboolean is_match;
 	Tpattern *pat;
-} Tpatmatch;
+} Tpatmatch;*/
 
 /***************************************************************
 how it works:
@@ -105,9 +137,6 @@ how it works:
 	the end value is re-evaluated once another pattern has gone beyond
 	it's value.
 	Once a match is found, the text is tagged with the right style, and
-	
-
-
 ***************************************************************/
 
 /*********************************/
@@ -124,7 +153,8 @@ static void print_meta_for_pattern(Tpattern *pat) {
 	GList *tmplist = g_list_first(highlight.all_highlight_patterns);
 	while (tmplist) {
 		if (((Tmetapattern *)tmplist->data)->pat == pat) {
-			g_print("pattern %p: filetype=%s, name=%s, num_childs=%d, numsub=%d, mode=%d\n", pat, ((Tmetapattern *)tmplist->data)->filetype, ((Tmetapattern *)tmplist->data)->name, g_list_length(pat->childs), pat->numsub, pat->mode);
+/*			g_print("pattern %p: filetype=%s, name=%s, num_childs=%d, numsub=%d, mode=%d\n", pat, ((Tmetapattern *)tmplist->data)->filetype, ((Tmetapattern *)tmplist->data)->name, g_list_length(pat->childs), pat->numsub, pat->mode);*/
+			g_print("pattern %p: name=%s\n",pat,((Tmetapattern *)tmplist->data)->name);
 			return;
 		}
 		tmplist = g_list_next(tmplist);
@@ -143,6 +173,57 @@ static void print_meta_for_tag(GtkTextTag *tag) {
 }
 
 #endif /* DEBUG */
+
+#ifdef HL_PROFILING
+static void hl_profiling_reset(Tdocument *doc) {
+	/* reset all patterns for the current filetype */
+	GList *tmplist = g_list_first(highlight.all_highlight_patterns);
+	while (tmplist){
+		Tmetapattern *mpat = (Tmetapattern *)tmplist->data;
+		if (strcmp(mpat->filetype, doc->hl->type)==0) {
+			mpat->pat->runcount = 0;
+			mpat->pat->applycount = 0;
+			mpat->pat->total_ms = 0;
+		}
+		tmplist = g_list_next(tmplist);
+	}
+	times(&highlight.tms1);
+}
+
+static void hl_profiling_tagstart(Tpattern *pat) {
+	times(&pat->tms1);
+	pat->runcount++;
+}
+
+static void hl_profiling_tagstop(Tpattern *pat) {
+	times(&pat->tms2);
+	pat->total_ms += (glong) (double) ((pat->tms2.tms_utime - pat->tms1.tms_utime) * 1000 / sysconf(_SC_CLK_TCK));
+}
+
+static void hl_profiling_tagapply(Tpattern *pat) {
+	pat->applycount++;
+}
+
+static void hl_profiling_print(Tdocument *doc) {
+	GList *tmplist;
+	glong tot_ms;
+	/* print results for all patterns for the current filetype */
+	times(&highlight.tms2);
+	tot_ms = (glong) (double) ((highlight.tms2.tms_utime - highlight.tms1.tms_utime) * 1000 / sysconf(_SC_CLK_TCK));
+	g_print("PROFILING: total time %ld ms\n", tot_ms);
+	tmplist = g_list_first(highlight.all_highlight_patterns);
+	while (tmplist){
+		Tmetapattern *mpat = (Tmetapattern *)tmplist->data;
+		if (strcmp(mpat->filetype, doc->hl->type)==0) {
+			g_print("PROFILING: patterns %s ran %d times, %d times successful (%d%%), and was matching %ld ms\n", mpat->name, mpat->pat->runcount, mpat->pat->applycount,(int)(100.0*mpat->pat->applycount/mpat->pat->runcount),mpat->pat->total_ms);
+		}
+		tmplist = g_list_next(tmplist);
+	}
+	g_print("-- -- -- -- -- --\n");
+}
+
+#endif /* HL_PROFILING */
+
 #ifdef HL_TIMING
 typedef struct {
 	struct tms tms1;
@@ -182,14 +263,6 @@ static void timing_stop(gint id) {
 /* initializing the highlighting */
 /*********************************/
 
-static int pcre_compile_options(gint case_insens) {
-	if (case_insens) {
-		return PCRE_CASELESS|PCRE_MULTILINE;
-	} else {
-		return PCRE_MULTILINE;
-	}
-}
-
 static void compile_pattern(gchar *filetype, gchar *name, gint case_insens
 			, gchar *pat1, gchar *pat2, gint mode, gchar *parentmatch
 			, gchar *fgcolor, gchar *bgcolor, gint weight, gint style) {
@@ -214,7 +287,7 @@ static void compile_pattern(gchar *filetype, gchar *name, gint case_insens
 		}
 		break;
 	case 3:
-		if (!(pat1 && strlen(pat1) && atoi(pat1) > 0 && atoi(pat1) < NUM_SUBMATCHES)) {
+		if (!(pat1 && strlen(pat1) && atoi(pat1) > 0 && atoi(pat1) < MAX_OVECTOR)) {
 			g_print("error compiling pattern '%s' for mode 3: sub-pattern number missing, too large or incorrect\n", name);
 			return;
 		}
@@ -233,7 +306,9 @@ static void compile_pattern(gchar *filetype, gchar *name, gint case_insens
 			const char *err=NULL;
 			int erroffset=0;
 			DEBUG_MSG("compiling pat1 '%s'\n", pat1);
-			pat->reg1.pcre = pcre_compile(pat1, pcre_compile_options(case_insens),&err,&erroffset,NULL);
+			pat->reg1.pcre = pcre_compile(pat1,
+					(case_insens) ? PCRE_CASELESS|PCRE_MULTILINE|PCRE_DOTALL : PCRE_MULTILINE|PCRE_DOTALL,
+					&err,&erroffset,NULL);
 			if (err) {
 				g_print("error compiling pattern %s offset %d\n", err, erroffset);
 			}
@@ -242,12 +317,20 @@ static void compile_pattern(gchar *filetype, gchar *name, gint case_insens
 			if (err) {
 				g_print("error studying pattern %s\n", err);
 			}
+			if (pcre_fullinfo(pat->reg1.pcre,pat->reg1.pcre_e,PCRE_INFO_CAPTURECOUNT,&pat->ovector_size)!=0) {
+				g_print("error gettting info for pattern %s\n", pat1);
+			}
+			if (pat->ovector_size > MAX_OVECTOR) pat->ovector_size = MAX_OVECTOR;
+			if (pat->ovector_size < MIN_OVECTOR) pat->ovector_size = MIN_OVECTOR;
+			pat->ovector = g_malloc((pat->ovector_size+1)*3*sizeof(int));
 		}
 		if (mode == 1) {
 			const char *err=NULL;
 			int erroffset=0;
 			DEBUG_MSG("compiling pat2 '%s'\n", pat2);
-			pat->reg2.pcre = pcre_compile(pat2, pcre_compile_options(case_insens),&err,&erroffset,NULL);
+			pat->reg2.pcre = pcre_compile(pat2,
+					(case_insens) ? PCRE_CASELESS|PCRE_MULTILINE|PCRE_DOTALL : PCRE_MULTILINE|PCRE_DOTALL,
+					&err,&erroffset,NULL);
 			if (err) {
 				g_print("error compiling pattern %s offset %d\n", err, erroffset);
 			}
@@ -259,6 +342,8 @@ static void compile_pattern(gchar *filetype, gchar *name, gint case_insens
 		}
 		if (mode == 3) {
 			pat->numsub = atoi(pat1);
+			pat->ovector_size = 3;
+			pat->ovector = g_malloc((pat->ovector_size+1)*3*sizeof(int));
 		}
 
 		pat->tag = gtk_text_tag_new(NULL);
@@ -282,6 +367,7 @@ static void compile_pattern(gchar *filetype, gchar *name, gint case_insens
 				g_object_set(pat->tag, "style", PANGO_STYLE_ITALIC, NULL);
 			}
 		}
+		DEBUG_MSG("adding tag %p to table %p\n", pat->tag, highlight.tagtable);
 		gtk_text_tag_table_add(highlight.tagtable, pat->tag);
 		/* from the documentation:
 			When adding a tag to a tag table, it will be assigned the highest priority in the table by 
@@ -298,7 +384,9 @@ static void compile_pattern(gchar *filetype, gchar *name, gint case_insens
 			mpat->parentmatch = g_strdup("^top$");
 		}
 		mpat->filetype = g_strdup(filetype);
+		DEBUG_MSG("adding mpat %s (%p) to list %p\n", mpat->name, mpat, highlight.all_highlight_patterns);
 		highlight.all_highlight_patterns = g_list_append(highlight.all_highlight_patterns, mpat);
+		DEBUG_MSG("finished compiling pattern %s and added it to the list\n", mpat->name);
 	}
 }
 
@@ -316,14 +404,74 @@ static void add_patterns_to_parent(GList **list, gchar *filetype, gchar *name) {
 	}
 }
 
-void hl_init() {
+void filetype_highlighting_rebuild() {
 	GList *tmplist;
-	/* init main_v->filetypelist, the first set is the defaultset */
-	highlight.highlight_filetypes = NULL;
-	highlight.all_highlight_patterns = NULL; 
-	highlight.tagtable = gtk_text_tag_table_new();
 
-	/* start by initializing the types, they should come from the configfile */
+	/* remove filetypes from documents, but to reconnect them 
+	again after the rebuild, we temporary put a string with 
+	the filetype on that pointer */
+	if (main_v->documentlist) {
+		tmplist = g_list_first(main_v->documentlist);
+		while (tmplist) {
+			Tdocument *thisdoc = (Tdocument *)tmplist->data;
+			if (thisdoc->hl) {
+				DEBUG_MSG("doc %p has type %p named %s\n", thisdoc, thisdoc->hl, thisdoc->hl->type);
+				DEBUG_MSG("disconnected document %p from filetype %s\n", thisdoc, thisdoc->hl->type);
+				thisdoc->hl = g_strdup(thisdoc->hl->type);
+			}
+			tmplist = g_list_next(tmplist);
+		}
+	}
+
+	/* first remove the menu widgets, and delete the filetype structs */
+	DEBUG_MSG("filetype_highlighting_rebuild, testing for filetypelist existance\n");
+	if (main_v->filetypelist) {
+		tmplist = g_list_first(main_v->filetypelist);
+		while (tmplist) {
+			Tfiletype *filetype = (Tfiletype *)tmplist->data;
+			filetype_menu_destroy(filetype);
+			g_free(filetype->type);
+			g_strfreev(filetype->extensions);
+			g_free(filetype->update_chars);
+			if (filetype->icon) {
+				g_object_unref(filetype->icon);
+			}
+			/* the highlightpatterns are freed separately, see below */
+			g_free(filetype);
+			tmplist = g_list_next(tmplist);
+		}
+		g_list_free(main_v->filetypelist);
+		main_v->filetypelist = NULL;
+	}
+
+	DEBUG_MSG("filetype_highlighting_rebuild, testing for metapattern existence\n");
+	if (highlight.all_highlight_patterns) {
+		tmplist = g_list_first(highlight.all_highlight_patterns);
+		while (tmplist) {
+			Tmetapattern *mpat = tmplist->data;
+			pcre_free(mpat->pat->reg1.pcre);
+			pcre_free(mpat->pat->reg1.pcre_e);
+			pcre_free(mpat->pat->reg2.pcre);
+			pcre_free(mpat->pat->reg2.pcre_e);
+			g_free(mpat->pat->ovector);
+			gtk_text_tag_table_remove(highlight.tagtable,mpat->pat->tag);
+			g_free(mpat->pat);
+			g_free(mpat->name);
+			g_free(mpat->parentmatch);
+			g_free(mpat->filetype);
+			g_free(mpat);
+			tmplist = g_list_next(tmplist);
+		}
+		g_list_free(highlight.all_highlight_patterns);
+		highlight.all_highlight_patterns = NULL;
+	}
+
+	/* free other lists */
+	g_list_free(highlight.highlight_filetypes);
+	highlight.highlight_filetypes = NULL;
+	
+	DEBUG_MSG("filetype_highlighting_rebuild, rebuilding the filetype list\n");
+	/* now rebuild the filetype list */
 	tmplist = g_list_first(main_v->props.filetypes);
 	while (tmplist) {
 		gchar **strarr;
@@ -331,7 +479,9 @@ void hl_init() {
 		strarr = (gchar **) tmplist->data;
 		if (count_array(strarr) == 4) {
 			filetype = g_new(Tfiletype, 1);
+			filetype->menuitem = NULL;
 			filetype->type = g_strdup(strarr[0]);
+			DEBUG_MSG("extensions for %s loaded from %s\n", strarr[0], strarr[1]);
 			filetype->extensions = g_strsplit(strarr[1], ":", 127);
 			filetype->update_chars = g_strdup(strarr[2]);
 			if (strlen(strarr[3])){
@@ -351,12 +501,13 @@ void hl_init() {
 		}
 #ifdef DEBUG
 		else {
-			DEBUG_MSG("hl_init, filetype needs 4 params in array\n");
+			DEBUG_MSG("filetype_list_rebuild, filetype needs 4 params in array\n");
 		}
 #endif
 		tmplist = g_list_next(tmplist);
 	}
-	
+
+	DEBUG_MSG("filetype_highlighting_rebuild, compile configpatterns into metapatterns\n");
 	/* now compile the patterns in metapatterns, they should come from the configfile */
 	tmplist = g_list_first(main_v->props.highlight_patterns);
 	while (tmplist) {
@@ -368,7 +519,7 @@ void hl_init() {
 		}
 #ifdef DEBUG		
 		else {
-			g_print("hl_init, pattern needs 11 parameters in array\n");
+			g_print("filetype_list_rebuild, pattern %s does NOT have 11 parameters in array\n", strarr[0]);
 		}
 #endif
 		tmplist = g_list_next(tmplist);
@@ -378,6 +529,7 @@ void hl_init() {
 	
 	/* first compile the parentmatch pattern */
 	tmplist = g_list_first(highlight.all_highlight_patterns);
+	DEBUG_MSG("filetype_highlighting_rebuild, compile parentmatch pattern\n");
 	while (tmplist) {
 		const char *err=NULL;
 		int erroffset=0;
@@ -390,12 +542,14 @@ void hl_init() {
 	}
 	/* now match the top-level patterns */
 	tmplist = g_list_first(main_v->filetypelist);
+	DEBUG_MSG("filetype_highlighting_rebuild, match toplevel patterns\n");
 	while (tmplist) {
 		Tfiletype *filetype = (Tfiletype *)tmplist->data;
 		add_patterns_to_parent(&filetype->highlightlist, filetype->type, "top");
 		tmplist = g_list_next(tmplist);
 	}
 	/* now match the rest of the patterns */
+	DEBUG_MSG("filetype_highlighting_rebuild, match rest of the patterns\n");
 	tmplist = g_list_first(highlight.all_highlight_patterns);
 	while (tmplist) {
 		Tmetapattern *mpat = (Tmetapattern *)tmplist->data;
@@ -403,6 +557,7 @@ void hl_init() {
 		tmplist = g_list_next(tmplist);
 	}
 	/* free the parentmatch pattern now */
+	DEBUG_MSG("filetype_highlighting_rebuild, free-ing mpat->pcre\n");
 	tmplist = g_list_first(highlight.all_highlight_patterns);
 	while (tmplist) {
 		Tmetapattern *mpat = (Tmetapattern *)tmplist->data;
@@ -411,6 +566,7 @@ void hl_init() {
 	}
 	
 	/* now link the filetypes with highlight patterns to a new list */
+	DEBUG_MSG("filetype_highlighting_rebuild, rebuilding list of filetypes with highlighting patterns\n");
 	tmplist = g_list_first(main_v->filetypelist);
 	while (tmplist) {
 		if (((Tfiletype *)tmplist->data)->highlightlist) {
@@ -418,6 +574,33 @@ void hl_init() {
 		}
 		tmplist = g_list_next(tmplist);
 	}
+	
+	/* now we have finished the rebuilding of the filetypes, we 
+	have to connect all the documents with their filetypes again, we 
+	stored the name of the filetype temporary in the place of the Tfiletype,
+	undo that now */
+	if (main_v->documentlist) {
+		tmplist = g_list_first(main_v->documentlist);
+		while (tmplist) {
+			if (((Tdocument *)tmplist->data)->hl) {
+				gchar *tmpstr = (gchar *)((Tdocument *)tmplist->data)->hl;
+				((Tdocument *)tmplist->data)->hl = hl_get_highlightset_by_type(tmpstr);
+				DEBUG_MSG("reconnecting document %p to filetype %s\n", tmplist->data, tmpstr);
+				g_free(tmpstr);
+				((Tdocument *)tmplist->data)->need_highlighting = TRUE;
+			}
+			tmplist = g_list_next(tmplist);
+		}
+	}
+}
+
+void hl_init() {
+	/* init main_v->filetypelist, the first set is the defaultset */
+	highlight.highlight_filetypes = NULL;
+	highlight.all_highlight_patterns = NULL; 
+	highlight.tagtable = gtk_text_tag_table_new();
+
+	filetype_highlighting_rebuild();
 }
 
 /**************************************/
@@ -459,7 +642,7 @@ Tfiletype *hl_get_highlightset_by_filename(gchar * filename)
 	/* if none found return first set (is default set) */
 	tmplist = g_list_first(highlight.highlight_filetypes);
 	if (!tmplist) {
-		DEBUG_MSG("hl_get_highlightset_by_filename, no highlightsets?\n");
+		DEBUG_MSG("hl_get_highlightset_by_filename, no highlightsets? return NULL\n");
 		return NULL;
 	} else {
 		return (Tfiletype *) tmplist->data;
@@ -470,39 +653,83 @@ Tfiletype *hl_get_highlightset_by_filename(gchar * filename)
 /* applying the highlighting */
 /*****************************/
 
-static void patmatch_rematch(gboolean is_parentmatch, Tpatmatch *patmatch, gint offset, gchar *buf, gint length, Tpatmatch *parentmatch) {
+static void patmatch_init_run(GList *level) {
+	GList *tmplist = g_list_first(level);
+	while (tmplist) {
+		Tpattern * pat = (Tpattern *)tmplist->data;
+		memset(pat->ovector,0,sizeof(pat->ovector));
+		pat->is_match = FALSE;
+		patmatch_init_run(pat->childs);
+		tmplist = g_list_next(tmplist);
+	}
+}
+
+static void patmatch_rematch(gboolean is_parentmatch, Tpattern *pat, gint offset, gchar *buf, gint to, Tpattern *parentpat) {
 #ifdef DEBUG
-	DEBUG_MSG("patmatch_rematch, started for pat ");
-	print_meta_for_pattern(patmatch->pat);
+	if (to < offset) {
+		DEBUG_MSG("patmatch_rematch: impossible, to < offset, to=%d, offset=%d\n", to, offset);
+		exit(23);
+	}
+	if (to < 20) {
+		gchar *tmp = g_strndup(&buf[offset],to-offset);
+		DEBUG_MSG("patmatch_rematch, searching offset=%d,to=%d '%s' with pat ",offset,to,tmp);
+		g_free(tmp);
+	} else {
+		DEBUG_MSG("patmatch_rematch, searching offset=%d,to=%d with pat ",offset,to);
+	}
+	print_meta_for_pattern(pat);
 #endif
 	if (is_parentmatch) {
+#ifdef HL_PROFILING
+		hl_profiling_tagstart(pat);
+#endif
+	
 #ifdef HL_TIMING
 		timing_start(TIMING_PCRE_EXEC);
 #endif
-		patmatch->is_match = pcre_exec(patmatch->pat->reg2.pcre, patmatch->pat->reg2.pcre_e, buf, length, offset, 0, patmatch->ovector, NUM_SUBMATCHES);
+		pat->is_match = pcre_exec(pat->reg2.pcre, pat->reg2.pcre_e, buf, to, offset, 0, pat->ovector, pat->ovector_size);
 #ifdef HL_TIMING
 		timing_stop(TIMING_PCRE_EXEC);
+#endif
+#ifdef HL_PROFILING
+		hl_profiling_tagstop(pat);
 #endif
 	} else {
-		if (patmatch->pat->mode == 3) {
+		if (pat->mode == 3) {
 #ifdef DEBUG
-			DEBUG_MSG("patmatch_rematch, getting value from  parentmatch->ovector[%d]\n", patmatch->pat->numsub*2+1);
-			if ((patmatch->pat->numsub*2+1) >= NUM_SUBMATCHES) {
-				DEBUG_MSG("ovector out of bounds!!\n");
+			DEBUG_MSG("patmatch_rematch, getting value from  parentpat->ovector[%d]\n", pat->numsub*2+1);
+			if ((pat->numsub*2+1) >= MAX_OVECTOR || (pat->numsub*2+1) >=parentpat->ovector_size) {
+				DEBUG_MSG("wanted subpattern is out of bounds!!, parentpat->ovector_size=%d\n", parentpat->ovector_size);
 			} else {
-				DEBUG_MSG("parentmatch=%p\n",parentmatch);
+				DEBUG_MSG("parentpat=%p with ovector_size=%d,requested ovector segment=%d\n",parentpat,parentpat->ovector_size,pat->numsub*2+1);
 			}
 #endif
-			patmatch->ovector[0] = parentmatch->ovector[patmatch->pat->numsub*2];
-			patmatch->ovector[1] = parentmatch->ovector[patmatch->pat->numsub*2+1];
-			patmatch->is_match = TRUE;
+#ifdef HL_PROFILING
+			hl_profiling_tagstart(pat);
+#endif
+			pat->ovector[0] = parentpat->ovector[pat->numsub*2];
+			pat->ovector[1] = parentpat->ovector[pat->numsub*2+1];
+			pat->is_match = TRUE;
+#ifdef HL_PROFILING
+			hl_profiling_tagstop(pat);
+#endif
 		} else {
+#ifdef HL_PROFILING
+			hl_profiling_tagstart(pat);
+#endif
 #ifdef HL_TIMING
 			timing_start(TIMING_PCRE_EXEC);
 #endif
-			patmatch->is_match = pcre_exec(patmatch->pat->reg1.pcre, patmatch->pat->reg1.pcre_e, buf, length, offset, 0, patmatch->ovector, NUM_SUBMATCHES);
+			pat->is_match = pcre_exec(pat->reg1.pcre, pat->reg1.pcre_e, buf, to, offset, 0, pat->ovector, pat->ovector_size);
+			if (pat->is_match == -1) {
+				pat->ovector[0] = to;
+				pat->ovector[1] = to;
+			}
 #ifdef HL_TIMING
 			timing_stop(TIMING_PCRE_EXEC);
+#endif
+#ifdef HL_PROFILING
+			hl_profiling_tagstop(pat);
 #endif
 		}
 	}
@@ -513,7 +740,18 @@ static void applystyle(Tdocument *doc, gchar *buf, guint buf_char_offset, gint s
 	gint istart, iend;
 	guint char_start, char_end, byte_char_diff_start;
 
-	DEBUG_MSG("applystyle, coloring from so=%d to eo=%d\n", so, eo);
+#ifdef DEBUG
+	{
+		gchar *tmp;
+		DEBUG_MSG("applystyle, coloring from so=%d to eo=%d", so, eo);
+		tmp = g_strndup(&buf[so],eo-so);
+		DEBUG_MSG("'%s'\n", tmp);
+		g_free(tmp);
+	}
+#endif
+#ifdef HL_PROFILING
+	hl_profiling_tagapply(pat);
+#endif
 
 #ifdef HL_TIMING
 	timing_start(TIMING_UTF8);
@@ -526,7 +764,7 @@ static void applystyle(Tdocument *doc, gchar *buf, guint buf_char_offset, gint s
 #endif
 	istart = char_start+buf_char_offset;
 	iend = char_end+buf_char_offset;
-#ifdef HL_DEBUG
+#ifdef HL_DEBUG1
 	DEBUG_MSG("applystyle, byte_char_diff=%d\n", byte_char_diff_start);
 	DEBUG_MSG("applystyle, coloring from %d to %d\n", istart, iend);
 #endif
@@ -563,101 +801,110 @@ static void applystyle(Tdocument *doc, gchar *buf, guint buf_char_offset, gint s
  * parentmatch: if there is a parent with mode 1 we have to search for the end together with it's children
  * childs_list: a list of Tpattern that needs to be applied in this region
  */
-static void applylevel(Tdocument * doc, gchar * buf, guint buf_char_offset, gint offset, gint length, Tpatmatch *parentmatch, GList *childs_list) {
+
+static void applylevel(Tdocument * doc, gchar * buf, guint buf_char_offset, gint offset, gint to, Tpattern *parentpat, GList *childs_list) {
 	gint parent_mode1_start=offset;
 	gint parent_mode1_offset=offset;
-	if (parentmatch && parentmatch->pat->mode == 1) {
+	if (parentpat && parentpat->mode == 1) {
 		/* before any patmatch_rematch, this has the end of the start pattern */
-		parent_mode1_offset = parentmatch->ovector[1];
+		parent_mode1_offset = parentpat->ovector[1];
 	}
-	DEBUG_MSG("applylevel, started with offset=%d, length=%d\n", offset, length);
+	DEBUG_MSG("applylevel, started with offset=%d, to=%d\n", offset, to);
 	if (!childs_list) {
 		/* if the parent is mode 1 we still need to find the end for the parent */
-		if (parentmatch && parentmatch->pat->mode == 1) {
-			patmatch_rematch(TRUE, parentmatch, offset > parent_mode1_offset ? offset : parent_mode1_offset, buf, length, parentmatch);
-			if (parentmatch->is_match<1) {
-				DEBUG_MSG("no childs list, mode 1 parent has no match, matching to length %d\n", length);
-				parentmatch->ovector[1] = length;
-				parentmatch->is_match = 1;
+		if (parentpat && parentpat->mode == 1) {
+			patmatch_rematch(TRUE, parentpat, offset > parent_mode1_offset ? offset : parent_mode1_offset, buf, to, parentpat);
+			if (parentpat->is_match<1) {
+				DEBUG_MSG("no childs list, mode 1 parent has no match, matching to 'to' %d\n", to);
+				parentpat->ovector[1] = to;
+				parentpat->is_match = 1;
 			}
-			applystyle(doc, buf,buf_char_offset, offset, parentmatch->ovector[1], parentmatch->pat);
+			applystyle(doc, buf,buf_char_offset, offset, parentpat->ovector[1], parentpat);
 			DEBUG_MSG("no childs list, mode 1 parent is applied, returning\n");
 		} else {
 			DEBUG_MSG("no childs list, no mode 1 parent, returning\n");
 		}
 		return;
 	} else {
-		gint numpats, i;
-		Tpatmatch *patmatch;
-		/* create the patmatch structures for this level */
 		GList *tmplist;
-		gint lowest_pm = -2; /* -2 means no lowest match, -1 means the parents ending is the lowest match, 0... means some pattern is the lowest match */
-			i = 0;
-			numpats = g_list_length(childs_list);
-			patmatch = g_new(Tpatmatch, numpats + 1);
+		Tpattern *lowest_pm = NULL;
+#ifdef DEBUG
+			if (!childs_list) {
+				DEBUG_MSG("childs_list=%p\n",childs_list);
+			}
+#endif
 			tmplist = g_list_first(childs_list);
 			while (tmplist) {
-				patmatch[i].pat = (Tpattern *) tmplist->data;
-				/* match all patmatches for the first time and directly search for the lowest match */
-				patmatch_rematch(FALSE, &patmatch[i], offset, buf, length, parentmatch);
-				if ((patmatch[i].is_match > 0) && (lowest_pm == -2 || (patmatch[lowest_pm].ovector[0] > patmatch[i].ovector[0]))) {
-					lowest_pm = i;
+				Tpattern *pat = (Tpattern *) tmplist->data;
+				/* check if we need to match the pattern */
+				if (pat->ovector[0] <= offset) {
+					patmatch_rematch(FALSE, pat, offset, buf, to, parentpat);
+				}
+				if ((pat->is_match > 0) && (pat->ovector[1]<=to) &&(lowest_pm == NULL || (lowest_pm->ovector[0] > pat->ovector[0]))) {
+					lowest_pm = pat;
+#ifdef DEBUG
+					DEBUG_MSG("1 new lowest_pm=%p, pat->is_match=%d, start=%d, pattern=",lowest_pm,pat->is_match, pat->ovector[0]);
+					print_meta_for_pattern(lowest_pm);
+#endif
 				}
 #ifdef DEBUG
-				DEBUG_MSG("lowest_pm=%d, patmatch[%d].is_match=%d, start=%d, pattern=",lowest_pm,i,patmatch[i].is_match, patmatch[i].ovector[0]);
-				print_meta_for_pattern(patmatch[i].pat);
+				else {
+					DEBUG_MSG("1 pat %p is NOT the lowest pm with is_match=%d and ovector[0]=%d\n", pat, pat->is_match, pat->ovector[0]);
+				}
 #endif
 				tmplist = g_list_next(tmplist);
-				i++;
 			}
-			if (parentmatch && parentmatch->pat->mode == 1) {
+			if (parentpat && parentpat->mode == 1) {
 				/* the end of the parent pattern needs matching too */
-				DEBUG_MSG("matching the parent with offset %d\n", offset > parent_mode1_start ? offset : parent_mode1_start);
-				patmatch_rematch(TRUE, parentmatch, offset > parent_mode1_offset ? offset : parent_mode1_offset, buf, length, parentmatch);
-				if (!parentmatch->is_match) {
-					DEBUG_MSG("mode 1 parent has no match, matching to length %d\n", length);
-					parentmatch->ovector[1] = length;
-					parentmatch->ovector[0] = length;
-					parentmatch->is_match = 1;
+				DEBUG_MSG("matching the parent-end with offset %d\n", offset > parent_mode1_start ? offset : parent_mode1_start);
+				patmatch_rematch(TRUE, parentpat, offset > parent_mode1_offset ? offset : parent_mode1_offset, buf, to, parentpat);
+				if (parentpat->is_match <=0) {
+					DEBUG_MSG("mode 1 parent has no match, matching to 'to' %d\n", to);
+					parentpat->ovector[1] = to;
+					parentpat->ovector[0] = to;
+					parentpat->is_match = 1;
 				}
-				if (parentmatch->is_match && (lowest_pm == -2 || (patmatch[lowest_pm].ovector[0] > parentmatch->ovector[0]))) {
-					lowest_pm = -1;
+				if ((parentpat->is_match>0) && (lowest_pm == NULL || (lowest_pm->ovector[0] > parentpat->ovector[0]))) {
+					lowest_pm = parentpat;
+					DEBUG_MSG("3 lowest_pm=%p (parentpat!!), parentpat->is_match=%d, start=%d\n",lowest_pm,parentpat->is_match, parentpat->ovector[0]);
 				}
-				DEBUG_MSG("lowest_pm=%d, parentmatch->is_match=%d, start=%d\n",lowest_pm,parentmatch->is_match, parentmatch->ovector[0]);
-			}
-			
-			/* apply the lowest match */
-			while (lowest_pm > -2) {
-				if (lowest_pm == -1) {
 #ifdef DEBUG
-					DEBUG_MSG("parent is the match!! ");
-					print_meta_for_pattern(parentmatch->pat);
-					DEBUG_MSG("offset=%d, lowest_pm=%d with ovector[0]=%d, ovector[1]=%d\n", offset, lowest_pm, parentmatch->ovector[0],parentmatch->ovector[1]);
+				else {
+					DEBUG_MSG("3 (parent!!) is NOT the lowest pm with is_match=%d and ovector[0]=%d\n", parentpat->is_match, parentpat->ovector[0]);
+				}
+#endif
+			}
+		
+			/* apply the lowest match */
+			while (lowest_pm != NULL) {
+				if (lowest_pm == parentpat) {
+#ifdef DEBUG
+					DEBUG_MSG("*apply: offset=%d, lowest_pm=%p with ovector=%d, ovector[1]=%d\n", offset, lowest_pm, parentpat->ovector[0],parentpat->ovector[1]);
 #endif
 					/* apply the parent, and return from this level */
-					applystyle(doc, buf,buf_char_offset, parent_mode1_start, parentmatch->ovector[1], parentmatch->pat);
-					lowest_pm = -2; /* makes us return */
+					applystyle(doc, buf,buf_char_offset, parent_mode1_start, parentpat->ovector[1], parentpat);
+					lowest_pm = NULL; /* makes us return */
 				} else {
 #ifdef DEBUG
-					print_meta_for_pattern(patmatch[lowest_pm].pat);
-					DEBUG_MSG("a childs is the match, offset=%d, lowest_pm=%d with start=%d and ovector[1]=%d\n", offset, lowest_pm, patmatch[lowest_pm].ovector[0],patmatch[lowest_pm].ovector[1]);
+					DEBUG_MSG("*apply:a childs is the match, offset=%d, lowest_pm=%p with start=%d and ovector[1]=%d", offset, lowest_pm, lowest_pm->ovector[0],lowest_pm->ovector[1]);
+					print_meta_for_pattern(lowest_pm);
 #endif
-					switch (patmatch[lowest_pm].pat->mode) {
+					switch (lowest_pm->mode) {
 					case 1:
 						/* if mode==1 the style is applied within the applylevel for the children because the end is not yet 
 						known, the end is set in ovector[1] after applylevel for the children is finished */
-						applylevel(doc, buf,buf_char_offset, patmatch[lowest_pm].ovector[0], length, &patmatch[lowest_pm], patmatch[lowest_pm].pat->childs);
-						offset = patmatch[lowest_pm].ovector[1];
+						applylevel(doc, buf,buf_char_offset, lowest_pm->ovector[0], to, lowest_pm, lowest_pm->childs);
+						offset = lowest_pm->ovector[1];
 					break;
 					case 2:
-						applystyle(doc, buf,buf_char_offset, patmatch[lowest_pm].ovector[0], patmatch[lowest_pm].ovector[1], patmatch[lowest_pm].pat);
-						applylevel(doc, buf,buf_char_offset, patmatch[lowest_pm].ovector[0], patmatch[lowest_pm].ovector[1], &patmatch[lowest_pm], patmatch[lowest_pm].pat->childs);
-						offset = patmatch[lowest_pm].ovector[1];
+						applystyle(doc, buf,buf_char_offset, lowest_pm->ovector[0], lowest_pm->ovector[1], lowest_pm);
+						applylevel(doc, buf,buf_char_offset, lowest_pm->ovector[0], lowest_pm->ovector[1], lowest_pm, lowest_pm->childs);
+						offset = lowest_pm->ovector[1];
 					break;
 					case 3:
-						applystyle(doc, buf,buf_char_offset, patmatch[lowest_pm].ovector[0], patmatch[lowest_pm].ovector[1], patmatch[lowest_pm].pat);
-						applylevel(doc, buf,buf_char_offset, patmatch[lowest_pm].ovector[0], patmatch[lowest_pm].ovector[1], &patmatch[lowest_pm], patmatch[lowest_pm].pat->childs);
-						offset = patmatch[lowest_pm].ovector[1];
+						applystyle(doc, buf,buf_char_offset, lowest_pm->ovector[0], lowest_pm->ovector[1], lowest_pm);
+						applylevel(doc, buf,buf_char_offset, lowest_pm->ovector[0], lowest_pm->ovector[1], lowest_pm, lowest_pm->childs);
+						offset = lowest_pm->ovector[1];
 					break;
 					default:
 						/* unknown mode, cannot pass the pattern-compile-stage??? */
@@ -668,46 +915,64 @@ static void applylevel(Tdocument * doc, gchar * buf, guint buf_char_offset, gint
 					break;
 					}
 					
-					/* rematch the patterns that have a startpoint < offset */
-					lowest_pm = -2;
-					for (i = 0; i < numpats; i++) {
-						if (patmatch[i].ovector[0] < offset) {
-							if (patmatch[i].pat->mode == 3) {
-								patmatch[i].is_match = FALSE; /* mode 3 types can only match as first match */
+					/* init for the next round, rematch the patterns that have a startpoint < offset */
+					lowest_pm = NULL;
+					
+					tmplist = g_list_first(childs_list);
+					while (tmplist) {
+						Tpattern *pat = (Tpattern *) tmplist->data;
+						if ((pat->ovector[1] <=to) && (pat->ovector[0] <= offset)) {
+							if (pat->mode == 3) {
+								pat->is_match = FALSE; /* mode 3 types can only match as first match */
 							} else {
-								patmatch_rematch(FALSE, &patmatch[i], offset, buf, length, parentmatch);
-								DEBUG_MSG("rematch: cur lowest_pm=%d, patmatch[%d].is_match=%d\n",lowest_pm,i,patmatch[i].is_match);
+								patmatch_rematch(FALSE, pat, offset, buf, to, parentpat);
+								DEBUG_MSG("rematch: pat=%p, pat.is_match=%d, start=%d\n",pat,pat->is_match,pat->ovector[0]);
 							}
 						}
-						if ((patmatch[i].is_match>0) && (lowest_pm == -2 || (patmatch[lowest_pm].ovector[0] > patmatch[i].ovector[0]))) {
-							lowest_pm = i;
+						if ((pat->is_match>0) && (lowest_pm == NULL || (lowest_pm->ovector[0] > pat->ovector[0]))) {
+							lowest_pm = pat;
+							DEBUG_MSG("2 new lowest_pm=%p, start=%d\n",lowest_pm,lowest_pm->ovector[0]);
 						}
+#ifdef DEBUG
+						else {
+							DEBUG_MSG("2 pat %p is NOT the lowest pm with is_match=%d and ovector[0]=%d\n", pat, pat->is_match, pat->ovector[0]);
+						}
+#endif
+						tmplist = g_list_next(tmplist);
 					}
-					if (parentmatch && parentmatch->pat->mode == 1) {
+
+					if (parentpat && parentpat->mode == 1) {
 						/* the end of the parent pattern needs matching too */
-						if (parentmatch->ovector[0] < offset) {
-							patmatch_rematch(TRUE, parentmatch, offset, buf, length, parentmatch);
+						if (parentpat->ovector[0] < offset) {
+							patmatch_rematch(TRUE, parentpat, offset, buf, to, parentpat);
 						}
-						if (!parentmatch->is_match) {
-							DEBUG_MSG("mode 1 parent has no match, matching to length %d\n", length);
-							parentmatch->ovector[1] = length;
-							parentmatch->ovector[0] = length;
-							parentmatch->is_match = 1;
+						if (parentpat->is_match<=0) {
+							DEBUG_MSG("mode 1 parent has no match, matching to 'to'=%d\n", to);
+							parentpat->ovector[1] = to;
+							parentpat->ovector[0] = to;
+							parentpat->is_match = 1;
 						}
-						if (lowest_pm == -2 || (patmatch[lowest_pm].ovector[0] > parentmatch->ovector[0])) {
-							lowest_pm = -1;
+						if (lowest_pm == NULL || (lowest_pm->ovector[0] > parentpat->ovector[0])) {
+							lowest_pm = parentpat;
+							DEBUG_MSG("4 new lowest_pm=%p (parentmatch!!), start=%d\n",lowest_pm,lowest_pm->ovector[0]);
 						}
+#ifdef DEBUG
+						else {
+							DEBUG_MSG("4 (parent!!) is NOT the lowest pm with is_match=%d and ovector[0]=%d\n", parentpat->is_match, parentpat->ovector[0]);
+						}
+#endif
 					}
 #ifdef DEBUG
-					if (lowest_pm == -1) {
-						DEBUG_MSG("lowest_match %d has start %d\n", lowest_pm, parentmatch->ovector[0]);
-					} else {
-						DEBUG_MSG("lowest_match %d has start %d\n", lowest_pm, patmatch[lowest_pm].ovector[0]);
+					if (lowest_pm) {
+						if (lowest_pm == parentpat) {
+							DEBUG_MSG("lowest_match %p (parentpat) has start %d\n", lowest_pm, parentpat->ovector[0]);
+						} else {
+							DEBUG_MSG("lowest_match %p (some child) has start %d\n", lowest_pm, lowest_pm->ovector[0]);
+						}
 					}
 #endif
 				}
 		}
-		g_free(patmatch);
 	}
 }
 
@@ -725,17 +990,19 @@ void hl_reset_highlighting_type(Tdocument * doc, gchar * newfilename)
 		if (filetype != doc->hl) {
 			doc_remove_highlighting(doc);
 			doc->hl = filetype;
-			doc->need_highlighting = TRUE;
+			if (doc->highlightstate) {
+				doc->need_highlighting = TRUE;
+			}
 			menu_current_document_set_toggle_wo_activate(filetype, NULL);
 		}
 	}
 }
 
 gboolean hl_set_highlighting_type(Tdocument * doc, Tfiletype *filetype) {
-	if (filetype && filetype->highlightlist) {
+	if (filetype) {
+		doc_remove_highlighting(doc);
+		doc->hl = filetype;
 		if (filetype != doc->hl) {
-			doc_remove_highlighting(doc);
-			doc->hl = filetype;
 			doc->need_highlighting = TRUE;
 			menu_current_document_set_toggle_wo_activate(filetype, NULL);
 		}
@@ -750,12 +1017,19 @@ void doc_highlight_full(Tdocument * doc) {
 		return;
 	} else {
 		gchar *buf;
-		guint charcount = gtk_text_buffer_get_char_count(doc->buffer);
+		guint charcount;
+/*		GdkCursor *cursor = gdk_cursor_new(GDK_WATCH);
+		gdk_window_set_cursor(main_v->main_window->window, cursor);
+		gdk_window_set_cursor(main_v->current_document->view->window, cursor);
+		flush_queue();*/
+		charcount = gtk_text_buffer_get_char_count(doc->buffer);
 		doc_remove_highlighting(doc);
 #ifdef HL_TIMING
 		timing_init();
 #endif
-
+#ifdef HL_PROFILING
+		hl_profiling_reset(doc);
+#endif
 #ifdef HL_TIMING
 		timing_start(TIMING_TOTAL);
 #endif
@@ -766,6 +1040,7 @@ void doc_highlight_full(Tdocument * doc) {
 		g_assert(doc->hl);
 		g_assert(buf);
 #endif
+		patmatch_init_run(doc->hl->highlightlist);
 		applylevel(doc,buf,0,0,strlen(buf),NULL,doc->hl->highlightlist);
 		g_free(buf);
 #ifdef HL_TIMING
@@ -773,8 +1048,13 @@ void doc_highlight_full(Tdocument * doc) {
 		g_print("doc_highlight_full done, %ld ms total, %ld ms tagging (%dX), %ld ms matching (%dX)\n",timing[TIMING_TOTAL].total_ms, timing[TIMING_TEXTBUF].total_ms, timing[TIMING_TEXTBUF].numtimes, timing[TIMING_PCRE_EXEC].total_ms, timing[TIMING_PCRE_EXEC].numtimes);
 		g_print("%ld ms utf8-shit (%dX), %ld ms utf8-invalidate (%dX)\n", timing[TIMING_UTF8].total_ms, timing[TIMING_UTF8].numtimes, timing[TIMING_UTF8_INV].total_ms, timing[TIMING_UTF8_INV].numtimes);
 		g_print("%ld ms setting iters, %ld ms setting tags\n", timing[TIMING_TEXTBUF_ITER].total_ms, timing[TIMING_TEXTBUF_TAG].total_ms);
+#endif
+#ifdef HL_PROFILING
+		hl_profiling_print(doc);
 #endif
 		doc->need_highlighting = FALSE;
+/*		gdk_window_set_cursor(main_v->main_window->window, NULL);
+		gdk_cursor_unref(cursor);*/
 	}
 }
 
@@ -899,10 +1179,10 @@ void doc_highlight_line(Tdocument * doc)
 #endif					
 					if (pat && (pat->mode == 1)) {
 						gchar *string;
-						int ovector[NUM_SUBMATCHES];
+						int ovector[MAX_OVECTOR];
 						/* but first we do a quick test if the parent-pattern is indeed still valid */
 						string = gtk_text_buffer_get_text(doc->buffer, &itstart, &itend, FALSE);
-						if (0 < pcre_exec(pat->reg2.pcre, pat->reg2.pcre_e, string, strlen(string), 0, 0, ovector, NUM_SUBMATCHES)) {
+						if (0 < pcre_exec(pat->reg2.pcre, pat->reg2.pcre_e, string, strlen(string), 0, 0, ovector, MAX_OVECTOR)) {
 							/* the current line does not have the start of the tag or the end of the tag, but now 
 							   it does have a match on the end pattern --> so the pattern should be invalidated */
 							pat = NULL;
@@ -1004,6 +1284,9 @@ void doc_highlight_line(Tdocument * doc)
 	if (patternlist) {
 		gchar *buf;
 		DEBUG_MSG("doc_highlight_line from so=%d to eo=%d\n", so, eo);
+		
+		patmatch_init_run(doc->hl->highlightlist);
+		
 #ifdef HL_TIMING
 		timing_start(TIMING_TOTAL);
 #endif
@@ -1019,7 +1302,7 @@ void doc_highlight_line(Tdocument * doc)
 	}
 	doc->need_highlighting = FALSE;
 }
-
+#ifdef HIGHLIGHTING_DEFAULTS_NOW_EXTERNAL
 void hl_reset_to_default()
 {
 	gchar **arr;
@@ -1144,7 +1427,7 @@ void hl_reset_to_default()
 	main_v->props.highlight_patterns = g_list_append(main_v->props.highlight_patterns,array_from_arglist("python", "comment", "0", "#.*$", "", "2", "", "#AAAAAA", "", "1", "2",  NULL));
 	main_v->props.highlight_patterns = g_list_append(main_v->props.highlight_patterns,array_from_arglist("python", "braces", "0", "[{()}\\[\\]]", "", "2", "", "#000000", "", "2", "0",  NULL));
 }
-
+#endif /* HIGHLIGHTING_DEFAULTS_NOW_EXTERNAL */
 GtkTextTagTable *highlight_return_tagtable() {
 	return highlight.tagtable;
 }
