@@ -22,7 +22,7 @@
  */
 /* 
  * Changes by Antti-Juhani Kaijanaho <gaia@iki.fi> on 1999-10-20
- * $Id: html.c,v 1.10 2002/11/30 16:50:39 oli4 Exp $
+ * $Id: html.c,v 1.12 2003/02/13 08:26:22 oli4 Exp $
  */
 
 #include <gtk/gtk.h>
@@ -70,7 +70,7 @@ void insert_char_cb(GtkWidget * widget, gint data) {
 	"&ograve;", "&oacute;", "&ocirc;", "&otilde;", "&ouml;",
 	"&oslash;", "&ugrave;", "&uacute;", "&ucirc;", "&uuml;",
 	"&yacute;", "&yuml;", "&uml;", "&acute;", "&cedil;", "&cent;",
-	"&pound;", "&curren;", "&yen;", "&not;", "&mult;", "&divide;",
+	"&pound;", "&curren;", "&yen;", "&not;", "&times;", "&divide;",
 	"&plusmn;", "&lt;", "&gt;", "&sup1;", "&sup2;", "&sup3;",
 	"&frac14;", "&frac12;", "&frac34;", "&deg;", "&ETH;", "&THORN;",
 	"&eth;", "&thorn;", "&szlig;", "&micro;", "&nbsp;", "&sect;",
@@ -420,8 +420,8 @@ void insert_time_cb(GtkWidget * widget, gpointer data)
 	ok_b = bf_stock_ok_button(GTK_SIGNAL_FUNC(insert_time_callback), (gpointer) timeinsert);
 	gtk_window_set_default(GTK_WINDOW(timeinsert->dialog), ok_b);
 	cancel_b = bf_stock_cancel_button(GTK_SIGNAL_FUNC(insert_time_cancel), (gpointer) timeinsert);
-	gtk_box_pack_start(GTK_BOX(hbox), ok_b, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), cancel_b, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), ok_b, TRUE, TRUE, 0);
 
 	gtk_widget_show_all(timeinsert->dialog);
 }
