@@ -1,4 +1,4 @@
-/*#define DEBUG*/
+/* #define DEBUG */
 
 #include <gtk/gtk.h>
 #include <string.h> /* strcmp() */
@@ -10,6 +10,7 @@
 #include "pixmap.h"
 
 enum {
+	view_line_numbers,
 	filebrowser_show_hidden_files,
 	filebrowser_show_others_files,
 	filebrowser_show_backup_files,
@@ -19,7 +20,7 @@ enum {
 	editor_tab_width,	/* editor tabwidth */
 	tab_font_string,		/* notebook tabs font */
 	highlight_num_lines_count, /* number of lines to highlight in continous highlighting */	
-
+	defaulthighlight,		/* highlight documents by default */
 	transient_htdialogs,  /* set html dialogs transient ro the main window */
 	restore_dimensions,
 	left_panel_width,
@@ -41,10 +42,12 @@ enum {
 	clear_undo_on_save, 	/* clear all undo information on file save */
 	newfile_default_encoding,/* if you open a new file, what encoding will it use */
 	auto_set_encoding_meta,/* auto set metatag for the encoding */
+	auto_update_meta,
 	ext_browsers_in_submenu,
 	ext_commands_in_submenu,
 	ext_outputbox_in_submenu,
-
+	default_advanced_snr,
+	cont_highlight_full, 	/* if you want to highlight the full text or just the line */
 	/* not yet in use */
 	image_editor_cline, 	/* image editor commandline */
 	cfg_weblint_cline,	/* weblint command line */
@@ -68,8 +71,6 @@ enum {
 	force_def_style,  /* force white editor background */
 	drop_at_drop_pos, 	/* drop at drop position instead of cursor position */
 	link_management, 	/* perform link management */
-	defaulthighlight,		/* highlight documents by default */
-	cont_highlight_full, 	/* if you want to highlight the full text or just the line */
 	cont_highlight_update,	/* update the syntax highlighting continuous */
 	html_ver,
 	cust_menu, 		/* entries in the custom menu */
@@ -194,7 +195,9 @@ static void font_button_lcb(GtkWidget *wid, GtkWidget *entry) {
 	const gchar *fontname;
 	fsd = gtk_font_selection_dialog_new(_("Select font"));
 	fontname = gtk_entry_get_text(GTK_ENTRY(entry)); /* do NOT free, this is an internal pointer */
-	gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(fsd), fontname);
+	if (strlen(fontname)) {
+		gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(fsd), fontname);
+	}
 	g_signal_connect(GTK_OBJECT(fsd),"response",G_CALLBACK(font_dialog_response_lcb),entry);
 	gtk_window_set_transient_for(GTK_WINDOW(GTK_DIALOG(fsd)), GTK_WINDOW(gtk_widget_get_toplevel(entry)));
 	gtk_window_set_modal(GTK_WINDOW(GTK_DIALOG(fsd)), TRUE);
@@ -418,6 +421,8 @@ static void filetype_apply_changes(Tprefdialog *pd) {
 			}
 			retval = gtk_tree_model_iter_next(GTK_TREE_MODEL(pd->ftd.lstore),&iter);
 		}
+	} else {
+		DEBUG_MSG("filetype_apply_changes, no curstrarr, nothing to apply\n");
 	}
 }
 
@@ -462,13 +467,13 @@ static void create_filetype_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 		GtkTreeSelection *select;
 	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 
-		column = gtk_tree_view_column_new_with_attributes ("Filetype", renderer,"text",0,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Filetype"), renderer,"text",0,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Extensions", renderer,"text",1,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Extensions"), renderer,"text",1,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Update chars", renderer,"text",2,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Update chars"), renderer,"text",2,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Icon", renderer,"text",3,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Icon"), renderer,"text",3,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
@@ -624,11 +629,11 @@ static void create_filefilter_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 		GtkTreeSelection *select;
 	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 
-		column = gtk_tree_view_column_new_with_attributes ("Filefilter", renderer,"text",0,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Filefilter"), renderer,"text",0,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ffd.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Inverse filtering", renderer,"text",1,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Inverse filtering"), renderer,"text",1,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ffd.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Filetypes in filter", renderer,"text",2,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Filetypes in filter"), renderer,"text",2,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ffd.lview), column);
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
@@ -970,7 +975,7 @@ static void create_highlightpattern_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 		GtkTreeSelection *select;
 	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 
-		column = gtk_tree_view_column_new_with_attributes ("Pattern", renderer,"text", 0,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Pattern"), renderer,"text", 0,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->hpd.lview), column);
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
@@ -1075,7 +1080,7 @@ static void browsers_apply_changes(Tprefdialog *pd) {
 				tmplist = g_list_next(tmplist);
 			}
 		} else {
-			DEBUG_MSG("browsers_apply_changes, NO strarr!!\n");
+			DEBUG_MSG("browsers_apply_changes, strarr=NULL!!\n");
 		}
 	} else {
 		DEBUG_MSG("browsers_apply_changes, NO curstrarr!!\n");
@@ -1134,9 +1139,9 @@ static void create_browsers_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 		GtkTreeSelection *select;
 	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 
-		column = gtk_tree_view_column_new_with_attributes ("Browser", renderer,"text",0,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Browser"), renderer,"text",0,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->bd.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Command", renderer,"text",1,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Command"), renderer,"text",1,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->bd.lview), column);
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
@@ -1179,7 +1184,7 @@ static gchar **external_create_strarr(Tprefdialog *pd) {
 }
 
 static void externals_apply_changes(Tprefdialog *pd) {
-	DEBUG_MSG("browsers_apply_changes, started\n");
+	DEBUG_MSG("externals_apply_changes, started\n");
 	if (pd->ed.curstrarr) {
 		gchar **strarr;
 		strarr = external_create_strarr(pd);
@@ -1252,9 +1257,9 @@ static void create_externals_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 		GtkTreeSelection *select;
 	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 
-		column = gtk_tree_view_column_new_with_attributes ("Label", renderer,"text",0,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Label"), renderer,"text",0,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ed.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Command", renderer,"text",1,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Command"), renderer,"text",1,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ed.lview), column);
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
@@ -1409,19 +1414,19 @@ static void create_outputbox_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 		GtkTreeSelection *select;
 	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 
-		column = gtk_tree_view_column_new_with_attributes ("Name", renderer,"text",0,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Name"), renderer,"text",0,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->od.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Pattern", renderer,"text",1,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Pattern"), renderer,"text",1,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->od.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("File #", renderer,"text",2,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("File #"), renderer,"text",2,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->od.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Line #", renderer,"text",3,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Line #"), renderer,"text",3,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->od.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Output #", renderer,"text",4,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Output #"), renderer,"text",4,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->od.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Command", renderer,"text",5,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Command"), renderer,"text",5,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->od.lview), column);
-		column = gtk_tree_view_column_new_with_attributes ("Show all output", renderer,"text",6,NULL);
+		column = gtk_tree_view_column_new_with_attributes (_("Show all output"), renderer,"text",6,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->od.lview), column);
 
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
@@ -1497,10 +1502,15 @@ static void preferences_ok_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	string_apply(&main_v->props.editor_font_string, pd->prefs[editor_font_string]);
 	integer_apply(&main_v->props.editor_tab_width, pd->prefs[editor_tab_width], FALSE);
 	integer_apply(&main_v->props.word_wrap, pd->prefs[word_wrap], TRUE);
+	integer_apply(&main_v->props.view_line_numbers, pd->prefs[view_line_numbers], TRUE);
+	integer_apply(&main_v->props.defaulthighlight, pd->prefs[defaulthighlight], TRUE);
 	integer_apply(&main_v->props.highlight_num_lines_count, pd->prefs[highlight_num_lines_count], FALSE);
+	integer_apply(&main_v->props.cont_highlight_full, pd->prefs[cont_highlight_full], TRUE);
+
 	integer_apply(&main_v->props.lowercase_tags, pd->prefs[lowercase_tags], TRUE);
 	integer_apply(&main_v->props.allow_dep, pd->prefs[allow_dep], TRUE);
 	integer_apply(&main_v->props.xhtml, pd->prefs[xhtml], TRUE);
+	integer_apply(&main_v->props.auto_update_meta, pd->prefs[auto_update_meta], TRUE);
 	
 	string_apply(&main_v->props.newfile_default_encoding, GTK_COMBO(pd->prefs[newfile_default_encoding])->entry);
 	integer_apply(&main_v->props.auto_set_encoding_meta, pd->prefs[auto_set_encoding_meta], TRUE);
@@ -1521,7 +1531,7 @@ static void preferences_ok_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	integer_apply(&main_v->props.ext_commands_in_submenu, pd->prefs[ext_commands_in_submenu], TRUE);
 	integer_apply(&main_v->props.ext_outputbox_in_submenu, pd->prefs[ext_outputbox_in_submenu], TRUE);
 	integer_apply(&main_v->props.transient_htdialogs, pd->prefs[transient_htdialogs], TRUE);
-	
+	integer_apply(&main_v->props.default_advanced_snr, pd->prefs[default_advanced_snr], TRUE);
 	
 	integer_apply(&main_v->props.filebrowser_show_hidden_files, pd->prefs[filebrowser_show_hidden_files], TRUE);
 	integer_apply(&main_v->props.filebrowser_show_others_files, pd->prefs[filebrowser_show_others_files], TRUE);
@@ -1553,10 +1563,23 @@ static void preferences_ok_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	
 	free_arraylist(main_v->props.browsers);
 	main_v->props.browsers = pd->lists[browsers];
+	DEBUG_MSG("preferences_ok_clicked_lcb, new browsers list=%p\n", main_v->props.browsers);
 	
 	free_arraylist(main_v->props.external_commands);
 	main_v->props.external_commands = pd->lists[external_commands];
 	
+	free_arraylist(main_v->props.outputbox);
+	main_v->props.outputbox	 = pd->lists[outputbox];
+	
+	/* apply the changes to highlighting patterns and filetypes to the running program */
+	filetype_highlighting_rebuild();
+	filetype_menu_rebuild(NULL);
+	filebrowser_filters_rebuild();
+	
+	menu_outputbox_rebuild();
+	encoding_menu_rebuild();
+	external_menu_rebuild(); /* browsers is also rebuild here! */
+	
 	preferences_destroy_lcb(NULL, NULL, pd);
 }
 static void preferences_cancel_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
@@ -1600,19 +1623,22 @@ static void preferences_dialog() {
 	gtk_box_pack_start(GTK_BOX(dvbox), pd->noteb, TRUE, TRUE, 0);
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Editor"),014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Editor"),150));
 
 	frame = gtk_frame_new(_("Editor options"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
 	vbox2 = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox2);
+	
 	pd->prefs[editor_font_string] = prefs_string(_("Font"), main_v->props.editor_font_string, vbox2, pd, string_font);
 	pd->prefs[editor_tab_width] = prefs_integer(_("Tab width"), main_v->props.editor_tab_width, vbox2, pd, 1, 50);
 	pd->prefs[word_wrap] = boxed_checkbut_with_value(_("Word wrap default"), main_v->props.word_wrap, vbox2);
+	pd->prefs[view_line_numbers] = boxed_checkbut_with_value(_("Line numbers by default"), main_v->props.view_line_numbers, vbox2);
+	pd->prefs[defaulthighlight] = boxed_checkbut_with_value(_("Highlight syntax by default"), main_v->props.defaulthighlight, vbox2);
 	pd->prefs[highlight_num_lines_count] = prefs_integer(_("Highlight # lines"), main_v->props.highlight_num_lines_count, vbox2, pd, 1, 8);
-
+	pd->prefs[cont_highlight_full] = boxed_checkbut_with_value(_("Always highlight full document"), main_v->props.cont_highlight_full, vbox2);
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("HTML"),014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("HTML"),154));
 
 	frame = gtk_frame_new(_("HTML options"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1622,9 +1648,10 @@ static void preferences_dialog() {
 	pd->prefs[lowercase_tags] = boxed_checkbut_with_value(_("Lowercase HTML tags"), main_v->props.lowercase_tags, vbox2);
 	pd->prefs[allow_dep] = boxed_checkbut_with_value(_("Use deprecated tags (e.g. <font> and <nobr>)"), main_v->props.allow_dep, vbox2);
 	pd->prefs[xhtml] = boxed_checkbut_with_value(_("Use XHTML style tags (<br />)"), main_v->props.xhtml, vbox2);
+	pd->prefs[auto_update_meta] = boxed_checkbut_with_value(_("Automatic update author meta"), main_v->props.auto_update_meta, vbox2);
 	
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Files"),014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Files"),152));
 
 	frame = gtk_frame_new(_("Encoding"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1675,7 +1702,7 @@ static void preferences_dialog() {
 	pd->prefs[allow_multi_instances] = boxed_checkbut_with_value(_("Allow multi instances of a file"), main_v->props.allow_multi_instances, vbox2);
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("User interface"), 014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("User interface"), 156));
 
 	frame = gtk_frame_new(_("Dimensions"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1699,6 +1726,7 @@ static void preferences_dialog() {
 	
 	pd->prefs[tab_font_string] = prefs_string(_("Notebook tab font (leave empty for gtk default)"), main_v->props.tab_font_string, vbox2, pd, string_font);
 	pd->prefs[transient_htdialogs] = boxed_checkbut_with_value(_("Make HTML dialogs transient"), main_v->props.transient_htdialogs, vbox2);
+	pd->prefs[default_advanced_snr] = boxed_checkbut_with_value(_("Advanced search and replace by default"), main_v->props.default_advanced_snr, vbox2);
 
 	frame = gtk_frame_new(_("File browser"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1711,7 +1739,7 @@ static void preferences_dialog() {
 	pd->prefs[filebrowser_dir_icon] = prefs_string(_("Directory icon"), main_v->props.filebrowser_dir_icon, vbox2, pd, string_file);
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Images"), 014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Images"), 155));
 
 	frame = gtk_frame_new(_("Thumbnails"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1726,7 +1754,7 @@ static void preferences_dialog() {
 	}
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Filetypes"), 014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Filetypes"), 153));
 
 	frame = gtk_frame_new(_("Filetypes"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1743,7 +1771,7 @@ static void preferences_dialog() {
 	create_filefilter_gui(pd, vbox2);
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Syntax highlighting"), 014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Syntax highlighting"), 158));
 
 	frame = gtk_frame_new(_("Patterns"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1753,7 +1781,7 @@ static void preferences_dialog() {
 	create_highlightpattern_gui(pd, vbox2);
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("External programs"), 014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("External programs"), 151));
 
 	frame = gtk_frame_new(_("Browsers"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1770,7 +1798,7 @@ static void preferences_dialog() {
 	create_externals_gui(pd, vbox2);
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Output parsers"), 014));
+	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("Output parsers"), 157));
 	
 	frame = gtk_frame_new(_("Outputbox"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 5);
@@ -1789,11 +1817,12 @@ static void preferences_dialog() {
 
 		gtk_box_pack_start(GTK_BOX(dvbox), ahbox, FALSE, FALSE, 0);
 
+		but = bf_stock_cancel_button(GTK_SIGNAL_FUNC(preferences_cancel_clicked_lcb), pd);
+		gtk_box_pack_start(GTK_BOX(ahbox), but, TRUE, TRUE, 0);
+
 		but = bf_stock_ok_button(GTK_SIGNAL_FUNC(preferences_ok_clicked_lcb), pd);
 		gtk_box_pack_start(GTK_BOX(ahbox), but, TRUE, TRUE, 0);
 		gtk_window_set_default(GTK_WINDOW(pd->win), but);
-		but = bf_stock_cancel_button(GTK_SIGNAL_FUNC(preferences_cancel_clicked_lcb), pd);
-		gtk_box_pack_start(GTK_BOX(ahbox), but, TRUE, TRUE, 0);
 	}
 	gtk_widget_show_all(pd->win);
 }
